# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
import ddt

from lvestats.lib.commons import dateutil


@ddt.ddt
class TestParseDate(unittest.TestCase):
    CORRECT_PERIOD2 = (
        ("5s", 5),
        ("1m3m", 240),
        ('1m', 60),
        ('1h1m', 3660),
        ('2d13h1m5s', 219665)
    )

    WRONG_PERIOD2 = (
        "5",
        "0s",
        "0m",
        "1,5h",
        "0.7d",
        "6h7h9z"
    )

    def test_parse_ok_date(self):
        dt = dateutil.parse_date('2014-08-14 11:58')
        self.assertIsNotNone(dt)

    def test_bad_date(self):
        with self.assertRaises(ValueError):
            dateutil.parse_date('2014-08-14 11:5823r4234')

    @ddt.data(*CORRECT_PERIOD2)
    @ddt.unpack
    def test_parse_period2(self, test_data, test_result):
        self.assertEqual(dateutil.parse_period2(test_data), test_result)

    @ddt.data(*WRONG_PERIOD2)
    def test_parse_period2_ValueError(self, test_data):
        with self.assertRaises(ValueError):
            dateutil.parse_period2(test_data)

if __name__ == '__main__':
    unittest.main()
