# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest

import mock
from testfixtures import tempdir

from lvestats.orm import history
from lvestats.plugins.generic.cleaners import HistoryCleaner
from test.base.base_db_test import BaseDbTest

__author__ = 'shaman'


class TestHistoryCleaner(BaseDbTest):
    def setUp(self):
        BaseDbTest.setUp(self)
        self.cfg.update({
            'keep_history_days': 5,
            'server_id': 'server_1'})
        self.history_cleaner = HistoryCleaner()
        self.history_cleaner.set_db_engine(self.engine)
        self.history_cleaner.set_config(self.cfg)

    def set_now(self, now):
        self.history_cleaner.now = now

    def test_set_config(self):
        self.history_cleaner.set_config({"period": 1})
        self.assertEqual(self.history_cleaner.period, 60)
        self.history_cleaner.set_config({"period": 5})
        self.assertEqual(self.history_cleaner.period, 300)
        self.history_cleaner.set_config({'server_id': 'server_3'})
        self.assertEqual(self.history_cleaner.server_id, 'server_3')

    def test_cleanup(self):
        self.set_now(self.cfg['keep_history_days']*24*60*60 + 10)

        histories = [
            {'id': 1, 'server_id': 'server_1', 'created': 1},
            {'id': 2, 'server_id': 'server_1', 'created': 15},
            {'id': 0, 'server_id': 'server_2', 'created': 1},
            {'id': 1, 'server_id': 'server_2', 'created': 7},
            {'id': 2, 'server_id': 'server_2', 'created': 15},
            {'id': 1, 'server_id': 'server_3', 'created': 1}]
        self.engine.execute(history.__table__.insert(histories))
        self.assertEqual(6, len(list(self.engine.execute(history.__table__.select()))))

        with mock.patch('lvestats.plugins.generic.cleaners.reboot_lock'):
            self.history_cleaner.execute(lve_data={})
            self.assertEqual(5, len(list(self.engine.execute(history.__table__.select()))))

            self.history_cleaner.server_id = 'server_2'
            self.history_cleaner.execute(lve_data={})
            self.assertEqual(3, len(list(self.engine.execute(history.__table__.select()))))

            self.set_now(self.cfg['keep_history_days']*24*60*60 + 20)
            self.history_cleaner.execute(lve_data={})
            self.assertEqual(2, len(list(self.engine.execute(history.__table__.select()))))

    @tempdir()
    def test_clean_snapshots(self, snapshots_dir):
        """
        :type snapshots_dir: testfixtures.tempdirectory.TempDirectory
        """
        snapshots_dir.write('31416/9.snapshot', b'')
        snapshots_dir.write('31416/11.snapshot', b'')
        snapshots_dir.write('31417/8.snapshot', b'')
        snapshots_dir.write('31417/9.snapshot', b'')
        self.set_now(self.cfg['keep_history_days']*24*60*60 + 10)
        with mock.patch('lvestats.lib.snapshot.SNAPSHOT_PATH', snapshots_dir.path), \
                mock.patch('lvestats.plugins.generic.cleaners.reboot_lock'):
            self.history_cleaner.execute(lve_data={})
        snapshots_dir.compare(['31416/', '31416/11.snapshot'], recursive=True)


if __name__ == '__main__':
    unittest.main()
