# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
import random
from lvestats.lib.chart.rdp import ramerdouglas


def rnd():
    return (random.random() - .5) * .2


class TestDouglasPeucker(unittest.TestCase):

    def test_one_line(self):
        test_list = [(i, 0) for i in range(101)]
        self.assertListEqual(ramerdouglas(test_list, 0.1), [(0, 0), (100, 0)])

    def test_two_line(self):
        line1 = [(i, i) for i in range(100)]
        line2 = [(i + 100, 100 - i) for i in range(101)]
        two_line = line1 + line2
        self.assertListEqual(ramerdouglas(two_line, 0.1), [(0, 0), (100, 100), (200, 0)])

    def test_one_line_random(self):
        line = [(i, 0 + rnd()) for i in range(101)]
        result = ramerdouglas(line, 0.5)
        rounded_result = [(point[0], round(point[1])) for point in result]  # pylint: disable=round-builtin
        self.assertListEqual(rounded_result, [(0., 0.), (100., 0.)])

    def test_two_line_random(self):
        line1 = [(i, i + rnd()) for i in range(100)]
        line2 = [(i + 100, 100 - i + rnd()) for i in range(101)]
        two_line = line1 + line2
        result = ramerdouglas(two_line, 0.5)
        rounded_result = [(point[0], round(point[1])) for point in result]  # pylint: disable=round-builtin
        self.assertListEqual(rounded_result, [(0., 0.), (100., 100.), (200., 0.)])

    def test_one_line_long(self):
        line = [(i, 0 + rnd()) for i in range(1001)]
        result = ramerdouglas(line, 0.5)
        rounded_result = [(point[0], round(point[1])) for point in result]  # pylint: disable=round-builtin
        self.assertListEqual(rounded_result, [(0., 0.), (1000., 0.)])

    def test_two_line_long(self):
        line1 = [(i, i + rnd()) for i in range(1000)]
        line2 = [(i + 1000, 1000 - i + rnd()) for i in range(1001)]
        two_line = line1 + line2
        result = ramerdouglas(two_line, 0.5)
        rounded_result = [(point[0], round(point[1])) for point in result]  # pylint: disable=round-builtin
        self.assertListEqual(rounded_result, [(0., 0.), (1000., 1000.), (2000., 0.)])

    def test_recursion_depth_error(self):
        test_list = [(i, random.randint(0, 1)) for i in range(5000)]
        ramerdouglas(test_list, 0.7)


if __name__ == '__main__':
    unittest.main()
