# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
import datetime
import argparse
import ddt

import lvestats.lib.commons.argparse_utils as argparse_utils


@ddt.ddt
class TestArgparseUtils(unittest.TestCase):
    CORRECT_PERIOD_TYPE = (
        ("5", 300),
        ("65s", 65),
        ("1m3m", 240),
        ('1m', 60),
        ('1h1m', 3660),
        ('2d13h1m5s', 219665)
    )

    CORRECT_INT = ("1", "5", "1000", "1000000")
    CORRECT_FLOAT = ("0.0004", "0.9", "8.7", "1000000.01")
    NOT_A_NUMBER = ("q", "1234,9", "0e3", "@", "1/7")

    @ddt.data(*CORRECT_PERIOD_TYPE)
    @ddt.unpack
    def test_period_type(self, test_data, test_result):
        self.assertEqual(argparse_utils.period_type(test_data), test_result)

    def test_period_type2(self):
        self.assertEqual(argparse_utils.period_type2('1m', datetime.datetime(2014, 10, 8, 17, 53, 11, 6)),
                         (datetime.datetime(2014, 10, 8, 17, 52), datetime.datetime(2014, 10, 8, 17, 53)))

    def test_check_user_name(self):
        self.assertEqual(argparse_utils.check_user_name('root'), 0)
        with self.assertRaises(argparse.ArgumentTypeError):
            argparse_utils.check_user_name('roott')

    @ddt.data('100000000000000000', *(NOT_A_NUMBER + CORRECT_FLOAT))
    def test_check_from_zero_to_max_int_error(self, test_data):
        with self.assertRaises(argparse.ArgumentTypeError):
            argparse_utils.check_from_zero_to_max_int(test_data)

    @ddt.data('0', *CORRECT_INT)
    def test_check_from_zero_to_max_int_ok(self, test_data):
        self.assertEqual(argparse_utils.check_from_zero_to_max_int(test_data), int(test_data))

    @ddt.data('0', '1000000000000000000000', *NOT_A_NUMBER)
    def test_check_timestamp_error(self, test_data):
        with self.assertRaises(argparse.ArgumentTypeError):
            argparse_utils.check_timestamp(test_data)

    @ddt.data('1', *(CORRECT_INT + CORRECT_FLOAT))
    def test_check_timestamp_ok(self, test_data):
        self.assertEqual(argparse_utils.check_timestamp(test_data), float(test_data))

if __name__ == '__main__':
    unittest.main()
