# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import os

import mock

import lvestats.lib.commons.proctitle
from lvestats.lib.commons.func import get_domains
from test.base.base_db_test import BaseDbTest


class TestProctitle(BaseDbTest):
    get_domain_backup = None

    GET_ALL_DATA_RESULT = [
        ["1453390815", "test.com", "GET", "/", "HTTP/1.1", "123456"],
        ["1453390816", "test.com", "GET", "php-myadmin/index.php", "HTTP/1.1", "123457"],
        ["1453390817", "test.com", "GET", "/", "HTTP/1.1", "123458"],
        ["1453390818", "example.net", "POST", "php-myadmin/index.php", "HTTP/1.1", "123459"],
        ["1453390819", "example.net", "GET", "/", "HTTP/1.1", "123460"],
        ["1453390825", "example.com", "GET", "/test.html", "HTTP/1.0", "123461"],
        ["1453390835", "test.com", "POST", "/templates/siteground-j15-50/images/menu_row.jpg", "HTTP/1.1", "123462"],
        ["1453390845", "test.com", "GET", "/", "HTTP/1.1", "123463"],
        ["1453390855", "qwert.com", "POST", "/", "HTTP/1.1", "123464"],
        ["1453390865", "test34.com", "GET", "/", "HTTP/1.1", "123465"],
        ["1453390875", "test.com", "GET", "/project/arhiv/2011/2011_3(28).pdf", "HTTP/1.1", "123466"],
        ["1453390885", "example.net", "GET", "/project/arhiv/2011/2011_3(28).pdf", "HTTP/1.1", "123467"],
        ["1453390895", "test.com", "GET", "/project/arhiv/2011/2011_3(28).pdf", "HTTP/1.1", "123468"],
        ["1453390805", "test.com", "GET", "/", "HTTP/1.1", "123469"],
        ["1453390814", "example.net", "POST", "/templates/siteground-j15-50/images/menu_row.jpg", "HTTP/1.1", "123470"],
    ]

    @staticmethod
    def mock_domain(username):
        user_dict = {
            "mock": ["test.com"],
            "first": ["example.net"],
            "second": ["example.com"],
            "test_user": ["example.com"],
            "user7": ["example.net"],
            "qwert": [],
            "user10": ["qwerty.io"],
            "flaskbb": ["flaskbb.test247.cloudlinux.com"],
            "wordpress": ["wordpress.test247.cloudlinux.com"]
        }
        return user_dict[username]

    def setUp(self):
        BaseDbTest.setUp(self)
        shm_path = os.path.join(os.path.dirname(__file__), "data", "shm")
        self.proctitle = lvestats.lib.commons.proctitle.Proctitle(shm_path)

    def test_get_user_data(self):
        self.proctitle.now = 1453390913
        self.proctitle.parsed_data = self.GET_ALL_DATA_RESULT
        with mock.patch('lvestats.lib.commons.proctitle.get_all_user_domains', side_effect=self.mock_domain):
            self.assertEqual(len(self.proctitle.get_user_data("mock")), 8)
            self.assertEqual(len(self.proctitle.get_user_data("mock")[0]), 6)
            self.assertEqual(len(self.proctitle.get_user_data("first")), 4)
            self.assertEqual(len(self.proctitle.get_user_data("second")), 1)
            self.assertEqual(len(self.proctitle.get_user_data("test_user")), 1)
            self.assertEqual(len(self.proctitle.get_user_data("user7")), 4)
            self.assertEqual(len(self.proctitle.get_user_data("user10")), 0)
            self.assertEqual(len(self.proctitle.get_user_data("qwert")), 0)
            self.assertEqual(
                self.proctitle.get_user_data("second")[0],
                ['123461', 'example.com', 'GET', '/test.html', 'HTTP/1.0', '88.0']
            )

    def test_get_all_data(self):
        result = self.proctitle._get_all_data()
        expected_result = [
            ["1453981994", "flaskbb.test247.cloudlinux.com", "GET", "/category/1-test-category-1", "HTTP/1.0", "760686"],
            ["1453981982", "194.44.0.247", "GET", "/sl30_i.php", "HTTP/1.1", "760684"],
            ["1453981991", "flaskbb.test247.cloudlinux.com", "GET", "/category/1-test-category-1", "HTTP/1.0", "760687"],
            ["1453981988", "flaskbb.test247.cloudlinux.com", "GET", "/category/1-test-category-1", "HTTP/1.0", "764193"],
            ["1453981992", "wordpress.test247.cloudlinux.com", "GET", "/", "HTTP/1.0", "763986"]
        ]
        self.assertEqual(len(result), len(expected_result))
        self.assertListEqual(sorted(result), sorted(expected_result))

    def test_get_domains(self):
        with mock.patch("clcommon.cpapi.userdomains") as mock_domains, \
             mock.patch('lvestats.lib.commons.proctitle.get_all_user_domains', side_effect=self.mock_domain):
            mock_domains.return_value = [
                ('wordpress.test247.cloudlinux.com', '/home/wordpress/public_html'),
                ('xz.wordpress.test247.cloudlinux.com', '/home/wordpress/public_html/xz')
            ]
            self.assertEqual(
                sorted(get_domains("wordpress")),
                sorted(['wordpress.test247.cloudlinux.com', 'xz.wordpress.test247.cloudlinux.com'])
            )
            mock_domains.side_effect = IOError
            self.assertEqual(
                get_domains("wordpress", raise_exception=False),
                []
            )
