# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import os
from mock import patch

import lvestats.version
import unittest

import ddt
import mock

from lvestats.lib.commons import sentry

DDT_VERSION = (
    ("2.5-9.el7", "2.5-9", False, False),
    ("2.5-9.el15", "2.5-9", False, False),
    ("2.6-0.el6.15452.16.1485863269", "2.6-0", False, True),
    ("@VERSION@", "@VERSION@", False, True),
    ("2.5-9.el7", "2.5-9", True, True),
    ("2.5-9.el15", "2.5-9", True, True),
    ("2.6-0.el6.15452.16.1485863269", "2.6-0", True, True),
    ("@VERSION@", "@VERSION@", True, True))


@ddt.ddt
class TestSentry(unittest.TestCase):
    TEST_RECORD_LIST_FILE = os.path.join(
        os.path.abspath(os.path.dirname(__file__)), '_record_list')

    def setUp(self):
        self._version = lvestats.version.VERSION

    def tearDown(self):
        lvestats.version.VERSION = self._version

    @ddt.data(*DDT_VERSION)
    @ddt.unpack
    def test_version(self, version, release, path_exist, is_dev):
        with mock.patch("lvestats.version.VERSION", version):
            with mock.patch("os.path.exists", return_value=path_exist):
                client = sentry.init_sentry_client()
                self.assertEqual(client.release, release)
                self.assertEqual(client.tags["Developer"], is_dev)
                self.assertTrue(isinstance(client.tags["Control Panel Name"], str))

    def test_is_new_exception(self):
        if os.path.isfile(self.TEST_RECORD_LIST_FILE):
            os.unlink(self.TEST_RECORD_LIST_FILE)

        client = sentry.init_sentry_client()
        client._filter._record_list_file = self.TEST_RECORD_LIST_FILE

        # omit errors on one line in one file
        with patch('lvestats.lib.commons.sentry.traceback.extract_tb',
                   return_value=[['/some/path/to/file.py', 42, None, None]]):
            is_captured = client.is_new_exception(
                (ValueError, ValueError('some error happened here'), None))
            self.assertEqual(is_captured, True)

            is_captured = client.is_new_exception(
                (ValueError, ValueError('some error happened there'), None))
            self.assertEqual(is_captured, False)

            # some other exception on same line must be captured
            is_captured = client.is_new_exception(
                (KeyError, KeyError('some other error happened'), None))
            self.assertEqual(is_captured, True)

        # same error type, same file, but other line
        with patch('lvestats.lib.commons.sentry.traceback.extract_tb',
                   return_value=[['/some/path/to/file.py', 50, None, None]]):
            is_captured = client.is_new_exception(
                (ValueError, ValueError('some error happened here'), None))
            self.assertEqual(is_captured, True)
