# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import errno
import io
import unittest

import mock

from lvestats.lib.commons import server_status


class TestServerStatus(unittest.TestCase):
    def test_check_server_status(self):
        with mock.patch("builtins.open", side_effect=IOError):
            self.assertEqual(server_status.check_server_status(), server_status.STOPPED)
        with mock.patch("builtins.open", return_value=io.StringIO('35454545')):
            with mock.patch("os.kill", side_effect=OSError(errno.ESRCH, "No such process")):
                self.assertEqual(server_status.check_server_status(), server_status.DEAD)
        with mock.patch("builtins.open", return_value=io.StringIO('35454545')):
            with mock.patch("os.kill", side_effect=OSError(errno.EPERM, "Operation not permitted")):
                self.assertEqual(server_status.check_server_status(), server_status.RUNNING)
        with mock.patch("builtins.open", return_value=io.StringIO('35454545')):
            with mock.patch("os.kill", side_effect=None):
                self.assertEqual(server_status.check_server_status(), server_status.RUNNING)
