# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
import ddt

from lvestats.lib.commons.sizeutil import convert_bytes_for_graph, convert_bytes, \
    convert_powers_of_1000, convert_powers_of_1000_for_graph, dbgov_io_bytes_value


@ddt.ddt
class TestSizeutils(unittest.TestCase):
    """
    Tests for size utils functions
    """

    BYTES = [
        (123, "123B"),
        (1024, "1.0KB"),
        (12345, "12.1KB"),
        (12345678, "11.8MB"),
        (12545678, "12.0MB"),  # round
        (12525678, "11.9MB"),  # no round
        (12345678901, "11.5GB"),
        (12345678901234, "11.2TB")
    ]
    BYTES_FOR_GRAPH = [
        (123, "123B"),
        (1024, "1KB"),
        (1234, "1.2KB"),
        (12345, "12KB"),
        (1234567, "1.2MB"),
        (1795567, "1.7MB"),
        (12345678, "11MB"),
        (18595670, "17MB"),
        (1234567890, "1.1GB"),
        (12345678901, "11GB"),
        (1234567890123, "1.1TB"),
        (12345678901234, "11TB"),
    ]
    POWERS_OF_1000 = [
        (1, "1"),
        (1000, "1.0K"),
        (4000000, "4.0M"),
        (1234, "1.2K"),
        (12345, "12.3K"),
        (1234567, "1.2M"),
        (1795567, "1.8M"),
        (12345678, "12.3M"),
        (18595670, "18.6M"),
        (918595670, "918M"),
        (1234567890, "1.2G"),
        (12345678901, "12.3G"),
        (1234567890123, "1.2T"),
        (12345678901234, "12.3T"),
    ]
    POWERS_OF_1000_FOR_GRAPH = [
        (1, "1"),
        (1000, "1K"),
        (4000000, "4M"),
        (1234, "1.2K"),
        (12345, "12K"),
        (1234567, "1.2M"),
        (1795567, "1.8M"),
        (12345678, "12M"),
        (18595670, "18M"),
        (918595670, "918M"),
        (1234567890, "1.2G"),
        (12345678901, "12G"),
        (1234567890123, "1.2T"),
        (12345678901234, "12T"),
    ]

    @ddt.data(*BYTES)
    @ddt.unpack
    def test_convert_bytes(self, value, result):
        """
        Test convert bytes function
        """
        self.assertEqual(convert_bytes(value), result)

    @ddt.data(*BYTES_FOR_GRAPH)
    @ddt.unpack
    def test_convert_bytes_for_graph(self, value, result):
        """
        Test convert_bytes_for_graph function.
        Converting bytes to graph - always 2 digit value, int or float
        """
        self.assertEqual(convert_bytes_for_graph(value), result)

    @ddt.data(*POWERS_OF_1000)
    @ddt.unpack
    def test_convert_powers_of_1000(self, value, result):
        """
        Test convert_powers_of_1000 function.
        """
        self.assertEqual(convert_powers_of_1000(value), result)

    @ddt.data(*POWERS_OF_1000_FOR_GRAPH)
    @ddt.unpack
    def test_convert_powers_of_1000_for_graph(self, value, result):
        """
        Test convert_powers_of_1000_for_graph function.
        """
        self.assertEqual(convert_powers_of_1000_for_graph(value), result)

    @ddt.data(
        (0.5, 1, int(1.5 * 1024 * 1024)),
        (2.4, 1.2, int(3.6 * 1024 * 1024))
    )
    @ddt.unpack
    def test_dbgov_io_bytes_value(self, io_read_mb, io_write_mb, _expected_result):
        """
        Test for dbgov_io_bytes_value function
        """
        result = dbgov_io_bytes_value(io_read_mb, io_write_mb)
        self.assertEqual(result, _expected_result)


if __name__ == '__main__':
    unittest.main()
