# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import json
import os
import unittest

import mock

import lvestats.lib.info.lveinfomain
from test.helpers import temporary_user, capture_all
from lvestats.lib import config


class TestConfig(unittest.TestCase):

    def test_lvestats2_644(self):
        def clconfpars_load_644(location):
            if location.endswith('.readonly'):
                return {'conf': 'lvestats2.readonly'}
            else:
                return {'conf': 'lvestats2'}

        with mock.patch.object(config.clconfpars, 'load', clconfpars_load_644):
            self.assertDictEqual({'conf': 'lvestats2'}, config.read_config())

    def test_lvestats2_600(self):
        def clconfpars_load_600(location):
            if location.endswith('.readonly'):
                return {'conf': 'lvestats2.readonly'}
            else:
                raise IOError(13, "Permission denied: '{}'".format(location))

        with mock.patch.object(config.clconfpars, 'load', clconfpars_load_600):
            self.assertDictEqual({'conf': 'lvestats2.readonly'}, config.read_config())

    @unittest.skipUnless(os.getegid() == 0 and os.path.exists('/proc/lve/list'),
                         'Can run only on live test system')
    def test_check_output_json_as_user(self):
        """
        Try detect trash (logging or traceback) in json output
        :return:
        """
        conf_path = os.path.join(os.path.dirname(__file__), "data", "sysconfig", "lvestats2")
        conf_path_readonly = conf_path + '.readonly'
        test_user = 'test_lib_config'

        os.chmod(conf_path, 0o600)
        os.chmod(conf_path_readonly, 0o644)

        conf_load_backup = config.clconfpars.load

        def mock_load_config(location):
            if location.endswith('.readonly'):
                return conf_load_backup(conf_path_readonly)
            else:
                return conf_load_backup(conf_path)

        argv_ = ['--json', '--period', '10m', '--user', test_user,
                 '--compat', 'v2', '--style', 'user', '--show-all']
        test_function = lvestats.lib.info.lveinfomain.main
        with mock.patch.object(config.clconfpars, 'load', mock_load_config):
            with temporary_user(test_user):
                # run as test user
                with capture_all(test_function, config.read_config(), argv_=argv_) as output:
                    # check correct output
                    try:
                        json.loads(output["stdout"] + output["stderr"])
                    except ValueError:
                        self.fail(
                            "Output cannot be parsed as json:\n" +
                            output["stdout"] + output["stderr"])

if __name__ == '__main__':
    unittest.main()
