# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import os
import sys
import time
import unittest
from datetime import datetime

import test.helpers as helpers
import lvestats.lib.info.lveinfomain as lveinfomain
from lvestats.lib.commons import dateutil
from test.base.base_db_test import BaseDbTest


class TestLVEInfo(BaseDbTest):

    def setUp(self):
        BaseDbTest.setUp(self)
        os.environ['TZ'] = 'Europe/London'  # set the zero time zone
        time.tzset()

    def test_user_not_exists(self):
        try:
            with helpers.capture_all(
                    lveinfomain.main, self.cfg, self.engine, argv_=[
                        "--user", "no-such-user", "--style", "user", "--csv"]) as output:
                result = output["stdout"]
        except Exception as ex:
            self.fail(ex)
        self.assertTrue(result.count('\n') == 1)  # header only

    def test_main_daylight_saving_time(self):
        date_format = '%Y-%m-%d %H:%M:%S'
        time1 = dateutil.gm_datetime_to_unixtimestamp(
            datetime.strptime("2014-10-25 10:13:0", date_format))
        time2 = dateutil.gm_datetime_to_unixtimestamp(
            datetime.strptime("2014-10-25 10:15:0", date_format))
        time3 = dateutil.gm_datetime_to_unixtimestamp(
            datetime.strptime("2014-10-26 10:13:0", date_format))
        time4 = dateutil.gm_datetime_to_unixtimestamp(
            datetime.strptime("2014-10-26 10:15:0", date_format))
        time5 = dateutil.gm_datetime_to_unixtimestamp(datetime.now())

        # add server alive rows
        self.save_history_line('localhost', uid=0, _time=time1)
        self.save_history_line('localhost', uid=0, _time=time2)
        self.save_history_line('localhost', uid=0, _time=time3)
        self.save_history_line('localhost', uid=0, _time=time4)
        self.save_history_line('localhost', uid=0, _time=time5)

        self.save_history_line('localhost', uid=1, _time=time1, cpu=100)
        self.save_history_line('localhost', uid=1, _time=time2, cpu=100)
        self.save_history_line('localhost', uid=1, _time=time3, cpu=10)
        self.save_history_line('localhost', uid=2, _time=time4, cpu=50)
        self.save_history_line('localhost', uid=3, _time=time5, cpu=100)
        out = sys.stdout
        sys.stdout = open("/dev/null", "w")
        try:
            lveinfomain.main(self.cfg, self.engine, argv_=["-f", "2014-10-25", "-t", "2014-10-27"])
        except Exception as ex:
            sys.stdout = out
            self.fail(ex)
        else:
            sys.stdout = out

    def test_convert_to_dict(self):
        rows = [(0, 20, 500), (9999990, 20, 500)]
        converted_rows = lveinfomain._convert_to_dict(rows, ['ID', 'aEP', 'aCPU'])
        self.assertEqual(converted_rows, [{'aEP': 20, 'ID': 0, 'aCPU': 500}, {'aEP': 20, 'ID': 9999990, 'aCPU': 500}])

    def test_time_unit_without_id(self):
        err = sys.stderr
        sys.stderr = open("/dev/null", "w")
        with self.assertRaises(SystemExit) as cm:
            lveinfomain.main(self.cfg, self.engine, argv_=["--time-unit", "6"])
        sys.stderr = err
        self.assertEqual(cm.exception.code, 2)

        out = sys.stdout
        sys.stdout = open("/dev/null", "w")
        try:
            lveinfomain.main(dict(), self.engine, argv_=["--time-unit", "6", "--id", "1"])
        except Exception as ex:
            sys.stdout = out
            self.fail(ex)
        else:
            sys.stdout = out

    def test_wrong_db_type(self):
        """
        Just a smoke test. Passes if there is no exceptions
        """
        cfg = {
            'db_type': 'sqlite1111',
        }
        lveinfomain.main_(cfg)


if __name__ == '__main__':
    unittest.main()
