#!/usr/bin/python
# coding=utf-8
import mock
from unittest import TestCase, main
from mock import patch, Mock
from ddt import ddt, data, unpack

from lvestats.lib import uidconverter

__author__ = "Aleksandr Shyshatsky"


@ddt
class TestUidConverter(TestCase):
    def setUp(self):
        uidconverter._id_reseller_map_cache = None
        uidconverter._name_map_cache = None

        patcher1 = mock.patch('lvestats.lib.config.read_config', return_value={})
        patcher1.start()
        self.addCleanup(patcher1.stop)

    def test_get_name_map_cached(self):
        with \
                patch('lvestats.lib.uidconverter.NameMap.__init__',
                      return_value=None) as mock_name_map_init, \
                patch('lvestats.lib.uidconverter.NameMap.link_xml_node') as mock_link_xml_node, \
                patch('lvestats.lib.uidconverter._name_map_cache', None):
            for _ in range(3):
                uidconverter._get_name_map()
                self.assertEqual(mock_name_map_init.call_count, 1)
                self.assertEqual(mock_link_xml_node.call_count, 1)

    @data(
        [500, 'localid', 'localid', 'username_local'],
        [500, 'localid', 'otherid', 'username_db'],
        [1000000501, 'localid', 'localid', 'reseller_local'],
        [1000000502, 'localid', 'otherid', 'reseller_db'],
    )
    @unpack
    def test_uid_to_username(self, uid, local_server_id, server_id, result):
        with \
                patch('lvestats.lib.uidconverter._uid_to_reseller_local',
                      return_value='reseller_local'), \
                patch('lvestats.lib.uidconverter.uid_to_username_local',
                      return_value='username_local'), \
                patch('lvestats.lib.uidconverter._uid_to_reseller_db',
                      return_value='reseller_db'), \
                patch('lvestats.lib.uidconverter._uid_to_username_db',
                      return_value='username_db'):

            self.assertEqual(
                uidconverter.uid_to_username(uid, local_server_id, server_id, Mock()),
                result
            )

    def test_get_reseller_name_from_panel(self):
        with patch('lvestats.lib.uidconverter.NameMap.get_name', return_value=None), \
             patch('lvestats.lib.uidconverter.NameMap.link_xml_node'), \
             patch('pwd.getpwuid', side_effect=KeyError()), \
             patch('lvestats.lib.uidconverter.cpapi.get_reseller_id_pairs',
                   return_value={'reseller1': 1000}):
            self.assertEqual(uidconverter._uid_to_reseller_local(1000), 'reseller1')
            self.assertEqual(uidconverter._uid_to_reseller_local(2000), None)

    def test_disabled_reseller_name(self):
        with patch('lvestats.lib.uidconverter.NameMap.get_name', return_value=None), \
             patch('lvestats.lib.uidconverter.NameMap.link_xml_node'), \
             patch('pwd.getpwuid', return_value=Mock(pw_name="reseller2")):
            self.assertEqual(uidconverter._uid_to_reseller_local(1000), 'reseller2')

    @data(
        (1234,       1000001234),
        (1000001234, 1000001234),
    )
    @unpack
    def test_convert_id_to_lvp_id(self, some_id, output_):
        result = uidconverter.convert_id_to_lvp_id(some_id)
        self.assertEqual(result, output_)

    @staticmethod
    def _mock_reseller_to_uid(reseller):
        if reseller == 'resel1':
            return 1000001241

    @data(
        ('resel1', 1000001241),
        ('resel2', -1),
    )
    @unpack
    def test_convert_name_to_lvp_id(self, username, output_):
        with patch.object(uidconverter, 'reseller_to_uid',
                          self._mock_reseller_to_uid):
            result = uidconverter.convert_name_to_lvp_id(username)
            self.assertEqual(result, output_)


if __name__ == '__main__':
    main(verbosity=2)
