#!/usr/bin/python
# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
import ddt

from clcommon.cpapi.cpapiexceptions import NotSupported
from clcommon.utils import ExternalProgramFailed
from lvestats.lib.ustate import SQLSnapshot

# pylint: disable=line-too-long
LVEPS_OUTPUT_ONE = """        ID        EP       PNO            PID  TNO            TID  SPEED            MEM                                               COM
     510         1         3            ---    3            ---    27%              1                                               ---
                 ---       ---          13063  ---          13063    13%              1                                    stress --cpu 2
                 ---       ---          13062  ---          13062    13%              1                                    stress --cpu 2
                 ---       ---          13061  ---          13061     0%              1                                    stress --cpu 2

"""
LVEPS_OUTPUT_ONE2 = """      ID     EP    PNO        COM    TNO    TID    CPU      MEM         IO       IOPS
     500      0      3        ---      3    ---    25%        1          0        N/A
            ---    ---     md5sum    ---  45576    26%        1        N/A        N/A
            ---    ---       bash    ---  45523     0%        1        N/A        N/A
            ---    ---         su    ---  45522     0%        1        N/A        N/A
"""
LVEPS_OUTPUT_ONE3 = """      ID     EP    PNO        COM    TNO    TID    SPEED      MEM         IO       IOPS
     500      0      3        ---      3    ---    25%        1          0        N/A
            ---    ---     md5sum    ---  45576    26%        1        N/A        N/A
            ---    ---       bash    ---  45523     0%        1        N/A        N/A
            ---    ---         su    ---  45522     0%        1        N/A        N/A
"""
LVEPS_OUTPUT_INCORRECT_TID = """        ID        EP       PNO            PID  TNO            TID  SPEED            MEM                                               COM
     510         1         3            ---    3            ---    27%              1                                               ---
                 ---       ---          13063  ---          13063    13%              1                                    stress --cpu 2
                 ---       ---          13062  ---          805439140474983110649%              1                                    stress --cpu 2
                 ---       ---          13061  ---          13061     0%              1                                    stress --cpu 2

"""
LVEPS_OUTPUT_NONE = """      ID     EP    PNO        PID    TNO    TID    CPU      MEM         IO       IOPS
"""
LVEPS_OUTPUT_MULTI = """        ID        EP       PNO            PID  TNO            TID  SPEED            MEM                                               COM
     512         1         3            ---    3            ---    32%              1                                               ---
                 ---       ---          13097  ---          13097    13%              1                                    stress --cpu 2
                 ---       ---          13096  ---          13096    13%              1                                    stress --cpu 2
                 ---       ---          13095  ---          13095     0%              1                                    stress --cpu 2
     511         1         3            ---    3            ---    32%              1                                               ---
                 ---       ---          13094  ---          13094    13%              1                                    stress --cpu 2
                 ---       ---          13093  ---          13093    13%              1                                    stress --cpu 2
                 ---       ---          13090  ---          13090     0%              1                                    stress --cpu 2
     510         1         3            ---    3            ---    32%              1                                               ---
                 ---       ---          13089  ---          13089    13%              1                                    stress --cpu 2
                 ---       ---          13088  ---          13088    14%              1                                    stress --cpu 2
                 ---       ---          13085  ---          13085     0%              1                                    stress --cpu 2

"""
LVEPS_INCORRECT_OUTPUT = """        ID        EP       PNO            PID  TNO            TID  SPEED            MEM                                               COM
     512         1         3            ---    3            ---    32%              1                                               ---
                 ---       ---          13097  ---          13097    13%              1                                    stress --cpu 2
                 ---       ---          13096  ---          13096    13%              1                                    stress --cpu 2
                 ---       ---          13095  ---          13095     0%              1                                    stress --cpu 2
     511         1         3            ---    3            ---    32%              1                                               ---
                 ---       ---          13094  ---          13094    13%              1                                    stress --cpu 2
                 ---       ---          13093  ---          13093    13%              1                                    stress --cpu 2
                 ---       ---          13090  ---          13090     0%              1                                    stress --cpu 2
     510         1         3            ---    3            ---    32%              1                                               ---
     13%              1                                    stress --cpu 2
                 ---       ---          13088  ---          13088    14%              1                                    stress --cpu 2
                 ---       ---          13085  ---          13085     0%              1                                    stress --cpu 2
     513         1         3            ---    3            ---    32%              1                                               ---
                 ---       ---          13094  ---          13094    13%              1                                    stress --cpu 2
                 ---       ---          13093  ---          13093    13%              1                                    stress --cpu 2
                 ---       ---          13090  ---          13090     0%              1                                    stress --cpu 2
     514         1         3            ---    3            ---    32%              1                                               ---
                 ---       ---          13089  ---          13089    13%              1                                    stress --cpu 2
                 ---       ---          13088  ---          13088    14%              1                                    stress --cpu 2
                 ---       ---          13085  ---          13085     0%              1                                    stress --cpu 2
"""

lveps1 = """ID        EP       PNO            PID  TNO            TID  SPEED            MEM                                               COM
           510         0         2            ---    2            ---     0%              1                                               ---
                      ---       ---          17315  ---          17315     0%              1                                              bash
                      ---       ---          17313  ---          17313     0%              1                                                su"""

lveps2 = """ID        EP       PNO            PID  TNO            TID  SPEED            MEM                                               COM
           510         0         2            ---    2            ---     0%              1                                               ---
                      ---       ---          17315  ---          17315     0%              1                                             -bash
                     ---       ---          17313  ---          17313     0%              1                                      su - cltest1"""

expected_result1 = {
    510: {
        'CPU': '0%', 'MEM': '1', 'EP': '0', 'PNO': '2',
        'TID': {
            17313: {'CPU': '0%', 'MEM': '1', 'CMD': 'su', 'PID': '17313'},
            17315: {'CPU': '0%', 'MEM': '1', 'CMD': 'bash', 'PID': '17315'}},
        'TNO': '2'}}
expected_result2 = {
    510: {
        'CPU': '0%', 'MEM': '1', 'EP': '0', 'PNO': '2',
        'TID': {
            17313: {'CPU': '0%', 'MEM': '1', 'CMD': 'su - cltest1', 'PID': '17313'},
            17315: {'CPU': '0%', 'MEM': '1', 'CMD': '-bash', 'PID': '17315'}},
        'TNO': '2'}}
PARSED_ONE = {
    510: {
        'CPU': '27%', 'EP': '1', 'MEM': '1', 'PNO': '3',
        'TID': {
            13061: {'CMD': 'stress --cpu 2', 'CPU': '0%', 'MEM': '1', 'PID': '13061'},
            13062: {'CMD': 'stress --cpu 2', 'CPU': '13%', 'MEM': '1', 'PID': '13062'},
            13063: {'CMD': 'stress --cpu 2', 'CPU': '13%', 'MEM': '1', 'PID': '13063'}},
        'TNO': '3'}}
PARSED_ONE2 = {
    500: {
        'IO': '0', 'MEM': '1', 'CPU': '25%', 'EP': '0', 'IOPS': 'N/A', 'PNO': '3',
        'TID': {
            45576: {'IO': 'N/A', 'MEM': '1', 'CMD': 'md5sum', 'CPU': '26%', 'IOPS': 'N/A'},
            45522: {'IO': 'N/A', 'MEM': '1', 'CMD': 'su', 'CPU': '0%', 'IOPS': 'N/A'},
            45523: {'IO': 'N/A', 'MEM': '1', 'CMD': 'bash', 'CPU': '0%', 'IOPS': 'N/A'}},
        'TNO': '3'}}
PARSED_ONE3 = {
    500: {
        'IO': '0', 'MEM': '1', 'CPU': '25%', 'EP': '0', 'IOPS': 'N/A', 'PNO': '3',
        'TID': {
            45576: {'IO': 'N/A', 'MEM': '1', 'CMD': 'md5sum', 'CPU': '26%', 'IOPS': 'N/A'},
            45522: {'IO': 'N/A', 'MEM': '1', 'CMD': 'su', 'CPU': '0%', 'IOPS': 'N/A'},
            45523: {'IO': 'N/A', 'MEM': '1', 'CMD': 'bash', 'CPU': '0%', 'IOPS': 'N/A'}},
        'TNO': '3'}}
PARSED_INCORRECT_TID = {
    510: {
        'CPU': '27%', 'EP': '1', 'MEM': '1', 'PNO': '3',
        'TID': {
            13061: {'CMD': 'stress --cpu 2', 'CPU': '0%', 'MEM': '1', 'PID': '13061'},
            13063: {'CMD': 'stress --cpu 2', 'CPU': '13%', 'MEM': '1', 'PID': '13063'}},
        'TNO': '3'}}
PARSED_NONE = {}
PARSED_MULTI = {
    510: {'CPU': '32%',
          'EP': '1',
          'MEM': '1',
          'PNO': '3',
          'TID': {13085: {'CMD': 'stress --cpu 2',
                          'CPU': '0%',
                          'MEM': '1',
                          'PID': '13085'},
                  13088: {'CMD': 'stress --cpu 2',
                          'CPU': '14%',
                          'MEM': '1',
                          'PID': '13088'},
                  13089: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13089'}},
          'TNO': '3'},
    511: {'CPU': '32%',
          'EP': '1',
          'MEM': '1',
          'PNO': '3',
          'TID': {13090: {'CMD': 'stress --cpu 2',
                          'CPU': '0%',
                          'MEM': '1',
                          'PID': '13090'},
                  13093: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13093'},
                  13094: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13094'}},
          'TNO': '3'},
    512: {'CPU': '32%',
          'EP': '1',
          'MEM': '1',
          'PNO': '3',
          'TID': {13095: {'CMD': 'stress --cpu 2',
                          'CPU': '0%',
                          'MEM': '1',
                          'PID': '13095'},
                  13096: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13096'},
                  13097: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13097'}},
          'TNO': '3'}}
PARSED_INCORRECT_OUTPUT = {
    510: {'CPU': '32%',
          'EP': '1',
          'MEM': '1',
          'PNO': '3',
          'TID': {},
          'TNO': '3'},
    511: {'CPU': '32%',
          'EP': '1',
          'MEM': '1',
          'PNO': '3',
          'TID': {13090: {'CMD': 'stress --cpu 2',
                          'CPU': '0%',
                          'MEM': '1',
                          'PID': '13090'},
                  13093: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13093'},
                  13094: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13094'}},
          'TNO': '3'},
    512: {'CPU': '32%',
          'EP': '1',
          'MEM': '1',
          'PNO': '3',
          'TID': {13095: {'CMD': 'stress --cpu 2',
                          'CPU': '0%',
                          'MEM': '1',
                          'PID': '13095'},
                  13096: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13096'},
                  13097: {'CMD': 'stress --cpu 2',
                          'CPU': '13%',
                          'MEM': '1',
                          'PID': '13097'}},
          'TNO': '3'}}

TEST_DATA = [
    (LVEPS_OUTPUT_ONE, PARSED_ONE),
    (LVEPS_OUTPUT_ONE2, PARSED_ONE2),
    (LVEPS_OUTPUT_ONE3, PARSED_ONE3),
    (LVEPS_OUTPUT_NONE, PARSED_NONE),
    (LVEPS_OUTPUT_MULTI, PARSED_MULTI),
    (LVEPS_INCORRECT_OUTPUT, PARSED_INCORRECT_OUTPUT),
    (lveps1, expected_result1),
    (lveps2, expected_result2),
    (LVEPS_OUTPUT_INCORRECT_TID, PARSED_INCORRECT_TID)]


@ddt.ddt
class LVEpsCase(unittest.TestCase):

    def setUp(self):
        self.maxDiff = None

    @ddt.data(*TEST_DATA)
    @ddt.unpack
    def test01_pars_lveps(self, test_data, parsed_data):
        from lvestats.lib.ustate import _pars_lveps
        result = _pars_lveps(test_data)
        self.assertEqual(parsed_data, result)

    def test02_get_lveps(self):
        from lvestats.lib.ustate import get_lveps
        try:
            result = get_lveps()
        except ExternalProgramFailed:
            self.skipTest('lveps not installed')
        else:
            self.assertEqual(True, isinstance(result, dict))


class MysqlProcListCase(unittest.TestCase):
    def setUp(self):
        self.sql_snap = SQLSnapshot()

    def test_01_smooth(self):
        try:
            sql_snap = SQLSnapshot()
            sql_snap.connect()
            sql_snap.get()
            sql_snap.close()
        except NotSupported:
            self.skipTest('To run test need install some control panel')

    def test_02_cplogin_query(self):
        processlist = ((946,     'eximstats',      'localhost', 'eximstats',      'Sleep', 2993, '',                             None),
                       (2307765, 'integrat_test3', 'localhost', 'integrat_test3', 'Sleep', 0,    '',                             None),
                       (2315770, 'root',           'localhost', None,             'Sleep', 146,  '',                             None),
                       (2404334, 'root',           'localhost', None,             'Query', 0,    'init',                        'SHOW FULL PROCESSLIST'),
                       (2425288, 'integrat_test3', 'localhost', 'integrat_test3', 'Query', 0,    'Waiting for table level lock', "SELECT xxl FROM table1"))
        # mock method for obtain processlist
        self.sql_snap._raw_processlist = lambda: processlist

        def _refresh_map_mock():
            self.sql_snap._dblogin_cplogin_map = {'sexyfish_test1': 'sexyfish', 'sexyfish_test2': 'sexyfish', 'integrat_test3': 'integrator'}
        self.sql_snap._refresh_map = _refresh_map_mock
        snapshot = self.sql_snap.get(['integrator', 'some_other_user'])
        self.assertEqual(
            {'integrator': [['Query', 0, 'SELECT xxl FROM table1']]},
            snapshot)

    def test_03_cplogin_query(self):
        processlist = ((2425288, 'sexyfish_test1', 'localhost', 'sexyfish_test1', 'Query', 0,   '',                             "SELECT xxl FROM table1"),
                       (2425288, 'sexyfish_test2', 'localhost', 'sexyfish_test2', 'Query', 0,   '',                             "SELECT xxl FROM table1"),
                       (2404334, 'root',           'localhost',  None,            'Query', 0,   'init',                         'SHOW FULL PROCESSLIST'),
                       (2425288, 'integrat_test3', 'localhost', 'integrat_test3', 'Query', 0,   'Waiting for table level lock', "SELECT xxl FROM table1"))
        # mock method for obtain processlist
        self.sql_snap._raw_processlist = lambda: processlist

        def _refresh_map_mock():
            self.sql_snap._dblogin_cplogin_map = {'sexyfish_test1': 'sexyfish', 'sexyfish_test2': 'sexyfish', 'integrat_test3': 'integrator'}
        self.sql_snap._refresh_map = _refresh_map_mock
        snapshot = self.sql_snap.get(['integrator', 'sexyfish'])
        self.assertEqual(
            {'integrator': [['Query', 0, 'SELECT xxl FROM table1']],
             'sexyfish': [['Query', 0, 'SELECT xxl FROM table1'],
                          ['Query', 0, 'SELECT xxl FROM table1']]},
            snapshot)


if __name__ == '__main__':
    unittest.main()
