# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import logging
import unittest

import lvestats.lib.parsers.lve_create_db_argparse
from lvestats.lib.dbengine import validate_database
from lvestats.lib.commons.logsetup import setup_logging
from test.base.base_db_test import BaseDbTest
from lvestats.orm.history import history
from lvestats.lib.lve_create_db import _configure_db


class BaseTestLVEDB(BaseDbTest):
    def setUp(self):
        self.log = setup_logging(self.cfg, 'create_db', console_level=logging.INFO, file_level=logging.CRITICAL + 10)
        self.parser = lvestats.lib.parsers.lve_create_db_argparse.lve_create_db_parser()


class TestLVERecreateDB(BaseTestLVEDB):
    def test_recreate(self):
        result = validate_database(self.engine)
        self.assertFalse(result["table_error"])
        history.__table__.drop(self.engine)
        result = validate_database(self.engine)
        self.assertTrue(result["table_error"])
        _configure_db(
            configuration=self.cfg,
            engine=self.engine,
            log=self.log,
            opts=self.parser.parse_args(["--recreate"]))
        result = validate_database(self.engine)
        self.assertFalse(result["table_error"])


class TestLVECreateDB(BaseTestLVEDB):
    def test_create(self):
        result = validate_database(self.engine)
        self.assertFalse(result["table_error"])
        history.__table__.drop(self.engine)
        result = validate_database(self.engine)
        self.assertTrue(result["table_error"])
        _configure_db(
            configuration=self.cfg,
            engine=self.engine,
            log=self.log,
            opts=self.parser.parse_args([]))
        result = validate_database(self.engine)
        self.assertTrue(result["table_error"])


class TestLVEForceDB(BaseTestLVEDB):
    def test_force(self):
        result = validate_database(self.engine)
        self.assertFalse(result["table_error"])
        history.__table__.drop(self.engine)
        result = validate_database(self.engine)
        self.assertTrue(result["table_error"])
        _configure_db(
            configuration=self.cfg,
            engine=self.engine,
            log=self.log,
            opts=self.parser.parse_args(["--create-missing-tables"]))
        result = validate_database(self.engine)
        self.assertFalse(result["table_error"])
