# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest

from sqlalchemy import select

from test.base.base_db_test import BaseDbTest
from lvestats.lib import uidconverter
from lvestats.orm import user
from lvestats.plugins.generic.collectors import LVEUsernamesCollector
from lvestats.plugins.generic.dbsaver import DbUsernamesSaver

__author__ = 'shaman'


class TestLVEUsernamesCollector(unittest.TestCase):

    def test_disabled(self):
        lve_data = {}
        c = LVEUsernamesCollector()
        c.set_config({})
        c.execute(lve_data)

        self.assertEqual({}, lve_data)

    def test_enabled(self):
        lve_data = {}
        c = LVEUsernamesCollector()
        c.set_config({'collect_usernames': 'true',
                      'server_id': 'fooserver'})

        c.execute(lve_data)

        self.assertNotEqual({}, lve_data)
        # {'users': [(999, 'git', 'localhost'), (1001, 'boinc', 'localhost'), ...
        self.assertIsNotNone(lve_data.get('users', None))
        for user_info in lve_data['users']:
            self.assertIsInstance(user_info[0], int)
            self.assertIsInstance(user_info[1], str)
            self.assertEqual('fooserver', user_info[2])


class TestLVEUsernamesSaver(BaseDbTest):

    def test_save_collected_users(self):

        lve_data = {
            'users': [(111, 'bender', 'robohost'),
                      (222, 'farnsworth', 'planetexpress')]
        }

        c = DbUsernamesSaver()
        c.set_config({'collect_usernames': 'true'})
        c.set_db_engine(self.engine)

        c.execute(lve_data)

        users = self.engine.execute(user.__table__.select())
        self.assertEqual(2, len(list(users)))

        lve_data = {
            'users': [(111, 'bender', 'robohost'),
                      (222, 'farnsworth', 'planetexpress'),
                      (333, 'fry', 'planetexpress')]
        }

        c.execute(lve_data)

        users = list(self.engine.execute(select([user.uid, user.server_id, user.user_name])))
        self.assertEqual(3, len(users))

        for uid, server_id, user_name in users:
            self.assertTrue(uid in [111, 222, 333])
            self.assertTrue(server_id[:10] in ['robohost'[:10], 'planetexpress'[:10]], msg=server_id)
            self.assertTrue(user_name in ['bender', 'farnsworth', 'fry'])


class TestUidConverter(BaseDbTest):
    def setUp(self):
        BaseDbTest.setUp(self)
        self.engine.execute(user.__table__.insert(), {'uid': 111, 'user_name': 'bender', 'server_id': 'robohost'})

    def test_uid_converter_loads_uids_1(self):
        bender_uid = uidconverter.username_to_uid('bender', 'localhost', 'robohost', self.engine)
        self.assertEqual(111, bender_uid)

    def test_uid_converter_loads_uids_2(self):
        foo_uid = uidconverter.username_to_uid('foo', 'localhost', 'robohost', self.engine)
        self.assertIsNone(foo_uid)


if __name__ == '__main__':
    unittest.main()
