# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
from ddt import ddt, data, unpack
from datetime import datetime, timedelta

# import tested functions
from lvestats.lib.lveinfolib import dyn_time_unit_groups


@ddt
class DynTimeUnit(unittest.TestCase):

    @data(timedelta(hours=1), timedelta(hours=4), timedelta(days=1), timedelta(days=7), timedelta(days=30))
    def test_check_max_rows(self, period):
        period_to = datetime.now()
        time_unit_groups = dyn_time_unit_groups(period_to - period, period_to)
        # calculate output rows number
        rows_numb = sum([(_to - _from).total_seconds()//time_unit for _from, _to, time_unit in time_unit_groups])
        self.assertLess(rows_numb, 100)

    #      tested period          deviation
    @data([timedelta(minutes=10), timedelta(minutes=1)],
          [timedelta(minutes=30), timedelta(minutes=20)],
          [timedelta(hours=1),    timedelta(hours=2)],
          [timedelta(hours=4),    timedelta(hours=2)],
          [timedelta(days=1),     timedelta(days=2)],
          [timedelta(days=7),     timedelta(days=2)],
          [timedelta(days=30),    timedelta(days=2)]
          )
    @unpack
    def test_deviation(self, period, deviation):
        period_to = datetime.now()
        period_from = period_to - period
        time_unit_groups = dyn_time_unit_groups(period_from, period_to)
        time_unit_groups_sum = timedelta(0)
        for _from, _to, _ in time_unit_groups:
            time_unit_groups_sum += _to - _from

        self.assertLessEqual(period, time_unit_groups_sum)
        self.assertGreaterEqual(period + deviation, time_unit_groups_sum)


if __name__ == '__main__':
    unittest.main()
