# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import builtins
import io
import os
import sys
import unittest

import mock

import lvestats.lib.commons.logsetup as logsetup
import lvestats.lib.lvestats_server as lvestats_server

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
sys.path.append('/usr/share/lve-stats/')
sys.path.append(os.path.join(BASE_DIR, 'bin/usr_share/'))
open_original = builtins.open


def mock_file(filename, value):
    def open_function(*args):
        if args[0] == filename:
            return io.StringIO(value)
        else:
            open_file = open_original(*args)
            return open_file

    return open_function


class LvestatsServerTest(unittest.TestCase):
    def setUp(self):
        self.log = logsetup.setup_logging({})

    @mock.patch('lvestats.lib.lvestats_server.get_lve_version')
    def test_lve_version(self, mock_get_lve_version):
        mock_get_lve_version.return_value = 4
        with self.assertRaises(lvestats_server.EnvironmentException):
            lvestats_server._check_valid_lve_version(self.log)

        mock_get_lve_version.return_value = 6
        lvestats_server._check_valid_lve_version(self.log)


if __name__ == '__main__':
    unittest.main()
