#!/usr/bin/python
# coding=utf-8
import unittest
from email.mime.multipart import MIMEMultipart

import mock

from clcommon.mail_helper import MailSendFailed, MailHelper

__author__ = "Aleksandr Shyshatsky"


class TestStatsNotifierMailSender(unittest.TestCase):

    def setUp(self):
        self._mail_helper = MailHelper()

    @mock.patch('lvestats.plugins.generic.statsnotifier.MailHelper._sendmail_smtp')
    @mock.patch('lvestats.plugins.generic.statsnotifier.MailHelper._sendmail_console')
    def test_smtp_ok(self, mock_console_send, mock_smtp_send):
        self._mail_helper.sendmail('from@@localhost', ['to@localhost', 'to2@localhost'],
                                      MIMEMultipart('Message'))
        self.assertTrue(mock_smtp_send.called)
        self.assertFalse(mock_console_send.called)

    @mock.patch('lvestats.plugins.generic.statsnotifier.MailHelper._sendmail_smtp', side_effect=MailSendFailed(""))
    @mock.patch('lvestats.plugins.generic.statsnotifier.MailHelper._sendmail_console')
    def test_smtp_send_failed(self, mock_console_send, mock_smtp_send):
        self._mail_helper.sendmail('from@@localhost', ['to@localhost', 'to2@localhost'], MIMEMultipart('Message'))
        self.assertTrue(mock_smtp_send.called)
        self.assertTrue(mock_console_send.called)

    @mock.patch('lvestats.plugins.generic.statsnotifier.MailHelper._sendmail_smtp', side_effect=MailSendFailed(""))
    @mock.patch('lvestats.plugins.generic.statsnotifier.MailHelper._sendmail_console', side_effect=MailSendFailed(""))
    def test_smtp_error_console_error(self, mock_console_send, mock_smtp_send):
        self._mail_helper.sendmail('from@@localhost', ['to@localhost', 'to2@localhost'], MIMEMultipart('Message'))
        self.assertTrue(mock_smtp_send.called)
        self.assertTrue(mock_console_send.called)

if __name__ == '__main__':
    unittest.main(verbosity=1)
