# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import ddt
import mock
import unittest

from lvestats.core.plugin import LveStatsPlugin
from lvestats.plugins import default_plugins
from lvestats import main


class MockPlugin1(LveStatsPlugin):
    pass


class MockPlugin2(LveStatsPlugin):
    order = 500


class MockPluginLoader(object):
    plugins = [MockPlugin1, MockPlugin2]


@ddt.ddt
class TestMain(unittest.TestCase):
    def test_init_plugins_none(self):
        with mock.patch('lvestats.main.PluginLoader', return_value=MockPluginLoader()):
            plugins = main.init_plugins({'plugins': 'some_folder'})
        defaults = [p.__name__ for p in list(zip(*default_plugins()))[0]]
        self.assertEqual(len(plugins), len(default_plugins()) + 2)
        self.assertEqual([p.__name__ for p in plugins], ['MockPlugin2'] + defaults + ['MockPlugin1'])

    @ddt.data(
        [['NoSuchPlugin'], []],
        [
            ['CPUInfoCollector', 'CloudLinuxTopFileSaver', 'LVEUsageAnalyzer'],
            ['CPUInfoCollector', 'CloudLinuxTopFileSaver', 'LVEUsageAnalyzer']],
        [
            ['MockPlugin2', 'CPUInfoCollector', 'NoSuchPlugin', 'MockPlugin1'],
            ['MockPlugin2', 'CPUInfoCollector', 'MockPlugin1']])
    @ddt.unpack
    def test_init_user(self, test_data, result):
        with mock.patch('lvestats.main.PluginLoader', return_value=MockPluginLoader()):
            plugins = main.init_plugins({'plugins': 'some_folder'}, test_data)
        self.assertEqual([p.__name__ for p in plugins], result)
