# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import logging
import os
import shutil
import unittest

import testfixtures

from lvestats.core.plugin_loader import PluginLoader

MODULES = '/tmp/modules'
MODULES_REAL = MODULES + '_real'

__author__ = 'shaman'


class TestPluginLoader(unittest.TestCase):

    def setUp(self):
        shutil.rmtree(MODULES, ignore_errors=True)
        shutil.rmtree(MODULES_REAL, ignore_errors=True)
        try:
            os.unlink(MODULES)
        except:
            pass

    def make_module(self, module_name, module_text, container_simlink=False):
        if container_simlink:
            os.mkdir(MODULES_REAL)
            os.symlink(MODULES_REAL, MODULES)
        else:
            os.mkdir(MODULES)

        file_name = module_name+'.py'

        with open(os.path.join(MODULES, file_name), 'w') as f:
            f.write(module_text)
            f.close()

    def test_load_module(self):
        module_text = ('from lvestats.core.plugin import LveStatsPlugin\nclass TestModule(LveStatsPlugin):\n'
                       '    def __init__(self):\n'
                       '        pass\n'
                       '    def f1(self):\n'
                       '        return \'Hello world\'\n')

        self.make_module('m1', module_text)

        loader = PluginLoader(MODULES)
        plugins = loader.plugins
        hello_world = plugins[0]().f1()
        self.assertEqual(hello_world, 'Hello world')

    def test_incorrect_symlink(self):
        self.make_module('correct_module', 'from lvestats.core.plugin import LveStatsPlugin\n')
        os.symlink('/tmp/incorrect_symlink_bfru', os.path.join(MODULES, 'incorrect_symlink.py'))

        # temporary enable logging for capture log
        original_level = logging.root.manager.disable
        logging.disable(logging.DEBUG - 1)
        with testfixtures.LogCapture() as captured_logs:
            PluginLoader(MODULES)
        logging.disable(original_level)
        self.assertIn('WARNING\n  Incorrect symlink /tmp/modules/incorrect_symlink.py', str(captured_logs))

    def test_symlinkfolder(self):
        module_text = ('from lvestats.core.plugin import LveStatsPlugin\nclass TestModule(LveStatsPlugin):\n'
                       '    def __init__(self):\n'
                       '        pass\n'
                       '    def f1(self):\n'
                       '        return \'Hello world\'\n')

        self.make_module('m3', module_text, container_simlink=True)
        loader = PluginLoader(MODULES)
        plugins = loader.plugins

        self.assertEqual(len(plugins), 1)


if __name__ == '__main__':
    unittest.main()
