# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import ddt
import unittest
import lvestat

import lvestats.plugins.generic.analyzers as analyzers


@ddt.ddt
class TestLVEUsage(unittest.TestCase):
    @ddt.data(
        ({'memphy': 200}, False),
        ({'memphy': 200, "cpu_usage": 1}, True),
        ({'nproc': 1}, True),
        ({'io_usage': 10}, True),
        ({'iops': 1}, True),
        ({'cpu_fault': 1}, True),
        ({'memphy_fault': 1}, True),
        ({'mem_fault': 1}, True),
        ({'io_fault': 1}, True),
        ({'iops_fault': 1}, True),
        ({'mep_fault': 1}, True),
        ({'nproc_fault': 1}, True),
        ({'lcpu': 1000, 'lmemphy': 10, 'lmem': 1, 'io': 1, 'liops': 1, 'lep': 1, 'lnproc': 1}, False))
    @ddt.unpack
    def test_has_interesting_values(self, test_data, result):
        usage = analyzers.LVEUsage(8)
        for attr, value in list(test_data.items()):
            setattr(usage, attr, value)
        self.assertEqual(usage.has_interesting_values(), result)

    # TODO: uncomment after fixing LVES-734
    @ddt.data(
        (
            '503	0	45	25	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
            '503	0	45	25	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
            6, False),
        #(
        #    '503	0	45	25	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
        #    '503	0	50	25	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
        #    6, True),
        (
            '503	0	45	25	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
            '503	0	45	35	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
            6, True),
        (
            '503	0	45	25	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
            '503	1	45	25	22030	1	1	0	77	2	4	78	262144	100	0	0	0	0	0',
            6, False),
        (
            '503	0	45	25	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
            '503	0	45	25	22030	0	0	0	80	1	0	0	262144	100	0	0	0	0	0',
            6, True),
        (
            '503	0	45	25	2203	0	0	0	77	1	0	0	262144	100	0	0	0	0	0',
            '503	1	45	25	22030	1	1	1	77	2	4	78	262144	100	0	0	0	0	0',
            6, True),
        (
            '500	0	2500	0	100000	0	0	262144	20	1	0	0	262144	100	0	0	0	0	0	10	0',
            '500	1	2500	0	120000	2	1	262144	20	4	1	1	262144	100	0	10	1	1	1	10	7',
            8, False),
        #(
        #    '500	0	2500	0	100000	0	0	262144	20	1	0	0	262144	100	0	0	0	0	0	10	0',
        #    '500	1	3000	0	120000	2	1	262144	20	4	1	1	262144	100	0	10	1	1	1	10	7',
        #    8, True),
        (
            '500	0	2500	0	100000	0	0	262144	20	1	0	0	262144	100	0	0	0	0	0	10	0',
            '500	1	2500	10	120000	2	1	262144	20	4	1	1	262144	100	0	10	1	1	1	10	7',
            8, True))
    @ddt.unpack
    def test_has_changed_limits(self, first_data, second_data, lve_version, result):
        usage = analyzers.LVEUsage(lve_version)
        first_lvestats = lvestat.LVEStat(first_data, lve_version)
        second_lvestats = lvestat.LVEStat(second_data, lve_version)
        self.assertEqual(usage._has_changed_limits(first_lvestats, second_lvestats), result)
