# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import csv
import unittest

import mock

from lvestats.plugins.generic.aggregators import LveUsageAggregator5S
from lvestats.plugins.generic.analyzers import LVEUsageAnalyzer
from lvestats.plugins.generic.persistors import FileSaver
from test.test_data_validity import MockLVECollector


class TestLibSnapshot(unittest.TestCase):

    def setUp(self):
        patcher1 = mock.patch('lvestats.lib.config.read_config', return_value={})
        patcher1.start()
        self.addCleanup(patcher1.stop)

    @mock.patch('clcommon.clproc.ProcLve.resellers_supported', return_value=False)
    def test_execute(self, mock_resellers_supported):
        lve_data = {'totalHz': 10000, 'procs': 4, 'old_now': 1000}
        lve_collector_plugin = MockLVECollector()
        lveusage_analyzer_plugin = LVEUsageAnalyzer()
        lveusage_analyzer_plugin.now = 1100
        lveaggregator5s_plugin = LveUsageAggregator5S()
        file_saver_plugin = FileSaver('/tmp/info')
        lve_collector_plugin._proc_lve.mock_resellers_lines[0] = [
            '8:LVE	EP	lCPU	lIO	CPU	MEM	IO	lMEM	lEP	nCPU	fMEM	fEP	lMEMPHY	lCPUW	lNPROC	MEMPHY	fMEMPHY	NPROC	fNPROC  lIOPS   iOPS',
            '0	0	2000	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
            '500	0	2500	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
            '501	0	2500	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
            '1000	0	2000	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
            '111501	0	2500	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0']
        lve_collector_plugin.execute(lve_data)
        lve_collector_plugin._proc_lve.mock_resellers_lines[0] = [
            '8:LVE	EP	lCPU	lIO	CPU	MEM	IO	lMEM	lEP	nCPU	fMEM	fEP	lMEMPHY	lCPUW	lNPROC	MEMPHY	fMEMPHY	NPROC	fNPROC  lIOPS   iOPS',
            '0	0	2000	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
            '500	0	2500	1	1000001000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
            '501	0	2500	1	1000001000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
            '1000	0	2000	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
            '111501	0	2500	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0']
        lve_collector_plugin.execute(lve_data)
        lveusage_analyzer_plugin.execute(lve_data)
        lveaggregator5s_plugin.execute(lve_data)
        file_saver_plugin.execute(lve_data)
        with open('/tmp/info', 'r') as info_file:
            result = list(csv.reader(info_file, delimiter=','))

        # pylint: disable=line-too-long
        self.assertEqual(
            sorted(result),
            sorted([
                ['0', '0', '20', '0', '2000', '0', '4096', '0', '0', '4096', '0', '0', '0', '0', '0', '0', '0', '1', '10', '0', '0', '0', '0'],
                ['500', '0', '20', '40', '2500', '0', '4096', '0', '0', '4096', '0', '0', '0', '0', '0', '0', '0', '1', '10', '0', '0', '0', '0'],
                ['501', '0', '20', '40', '2500', '0', '4096', '0', '0', '4096', '0', '0', '0', '0', '0', '0', '0', '1', '10', '0', '0', '0', '0'],
                ['111501', '0', '20', '0', '2500', '0', '4096', '0', '0', '4096', '0', '0', '0', '0', '0', '0', '0', '1', '10', '0', '0', '0', '0']]))

    @mock.patch('clcommon.clproc.ProcLve.resellers_supported', return_value=True)
    @mock.patch('clcommon.clproc.ProcLve.lvp_id_list', return_value=[1005])
    def test_execute_lve10(self, mock_resellers_supported, mock_lvp_id_list):
        lve_data = {'totalHz': 10000, 'procs': 4, 'old_now': 1000}
        lve_collector_plugin = MockLVECollector()
        lveusage_analyzer_plugin = LVEUsageAnalyzer()
        lveusage_analyzer_plugin.now = 1100
        lveaggregator5s_plugin = LveUsageAggregator5S()
        file_saver_plugin = FileSaver('/tmp/info')
        # pylint: disable=line-too-long
        lve_collector_plugin._proc_lve.mock_resellers_lines[0] = [
            '10:LVE	lCPU	lCPUW	nCPU	lEP	lNPROC	lMEM	lMEMPHY	lIO	lIOPS	lNETO	lNETI	EP	CPU	MEM	IO	fMEM	fEP	MEMPHY	fMEMPHY	NPROC	fNPROC	IOPS	NETO	NETI',
            '0,0		10000	100	1	20	100	0	262144	1024	1024	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0',
            '0,4294967295		0	0	0	0	0	0	0	0	0	0	0	0	0	202346	0	0	0	0	0	171	0	0	0	0',
            '0,1003		10000	100	1	20	100	0	262144	1024	1024	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0']

        # diff from default:
        # 1004: lNPROC
        # 1000: lCPU
        # 1005: lCPU
        # pylint: disable=line-too-long
        lve_collector_plugin._proc_lve.mock_resellers_lines[1005] = [
            '10:LVE	lCPU	lCPUW	nCPU	lEP	lNPROC	lMEM	lMEMPHY	lIO	lIOPS	lNETO	lNETI	EP	CPU	MEM	IO	fMEM	fEP	MEMPHY	fMEMPHY	NPROC	fNPROC	IOPS	NETO	NETI',
            '1005,0		10000	100	1	20	100	0	262144	1024	1024	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0',
            '1005,4294967295		10000	100	1	20	100	0	262144	1024	1024	0	0	0	15558652504168	0	0	0	0	0	0	0	0	0	0	0',
            '1005,1000		1000	100	1	20	3	0	262144	1024	1024	0	0	0	15558652504168	221	1900	0	0	320	0	2	0	153	0	0',
            '1005,1004		10000	100	1	20	3	0	262144	1024	1024	0	0	0	15558652504168	221	1900	0	0	320	0	2	0	153	0	0',
            '1005,1005		1000	100	1	20	100	0	262144	1024	1024	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0']
        lve_collector_plugin.execute(lve_data)
        # diff:
        # 1003: lCPUW
        # pylint: disable=line-too-long
        lve_collector_plugin._proc_lve.mock_resellers_lines[0] = [
            '10:LVE	lCPU	lCPUW	nCPU	lEP	lNPROC	lMEM	lMEMPHY	lIO	lIOPS	lNETO	lNETI	EP	CPU	MEM	IO	fMEM	fEP	MEMPHY	fMEMPHY	NPROC	fNPROC	IOPS	NETO	NETI',
            '0,0		10000	100	1	20	100	0	262144	1024	1024	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0',
            '0,4294967295		0	0	0	0	0	0	0	0	0	0	0	0	0	202346	0	0	0	0	0	171	0	0	0	0',
            '0,1003		1000	100	1	20	100	0	262144	1024	1024	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0']
        # diff:
        # 1005: lCPUW changed
        # pylint: disable=line-too-long
        lve_collector_plugin._proc_lve.mock_resellers_lines[1005] = [
            '10:LVE	lCPU	lCPUW	nCPU	lEP	lNPROC	lMEM	lMEMPHY	lIO	lIOPS	lNETO	lNETI	EP	CPU	MEM	IO	fMEM	fEP	MEMPHY	fMEMPHY	NPROC	fNPROC	IOPS	NETO	NETI',
            '1005,0		10000	100	1	20	100	0	262144	1024	1024	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0',
            '1005,4294967295		10000	100	1	20	100	0	262144	1024	1024	0	0	0	15558652504168	0	0	0	0	0	0	0	0	0	0	0',
            '1005,1000		1000	100	1	20	3	0	262144	1024	1024	0	0	0	15558652504168	221	1900	0	0	320	0	2	0	153	0	0',
            '1005,1004		10000	100	1	20	100	0	262144	1024	1024	0	0	0	15558652504168	221	1900	0	0	320	0	2	0	153	0	0',
            '1005,1005		10000	100	1	20	100	0	262144	1024	1024	0	0	0	0	0	0	0	0	0	0	0	0	0	0	0']
        lve_collector_plugin.execute(lve_data)
        lveusage_analyzer_plugin.execute(lve_data)
        lveaggregator5s_plugin.execute(lve_data)
        file_saver_plugin.execute(lve_data)
        with open('/tmp/info', 'r') as info_file:
            result = list(csv.reader(info_file, delimiter=','))

        # pylint: disable=line-too-long
        self.assertEqual(
            sorted(result),
            sorted([
                ['0', '0', '20', '0', '10000', '0', '0', '0', '0', '262144', '0', '0', '100', '0', '0', '0', '0', '1024', '1024', '0', '0', '0', '0'],
                ['1000', '0', '20', '0', '1000', '221', '0', '0', '0', '262144', '320', '0', '3', '2', '0', '0', '0', '1024', '1024', '0', '0', '0', '0'],
                ['1004', '0', '20', '0', '10000', '221', '0', '0', '0', '262144', '320', '0', '100', '2', '0', '0', '0', '1024', '1024', '0', '0', '0', '0'],
                ['1003', '0', '20', '0', '1000', '0', '0', '0', '0', '262144', '0', '0', '100', '0', '0', '0', '0', '1024', '1024', '0', '0', '0', '0']
            ]))
