# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
import platform

import json
from lvestat import LVEStat
from lvestats.plugins.other.json_stats_saver import LVEStatsData


class TestLVEStatsData(unittest.TestCase):
    """
    Test cases for lve_stats_data.
    """
    maxDiff = None

    def setUp(self):
        self.t = LVEStatsData('/tmp/lve-stats/output')

    def test_empty_lve_data_set(self):
        out = self.t.format_json({})
        self.assertEqual(json.loads(out), json.loads('{ "hostname": "%s", "timestamp": "", "data": [  ] }' % platform.node()))

    def test_single_lve_data_set(self):
        test_data = LVEStat(None, 10)
        test_data.id = 11235813
        test_data.cpu_usage = 23581321
        test_usage = {11235813: test_data}
        out = self.t.format_json(test_usage)

        expected_out = '{ "hostname": "%s", "timestamp": "", ' % platform.node()
        expected_out += '"data": [ { "id": "11235813", "cpu": "0", "cpu_usage": "23581321", '
        expected_out += '"io": "0", "io_usage": "0", "iops": "0", "lcpuw": "0", "lep": "0", "liops": "0", '
        expected_out += '"lmem": "0", "lmemphy": "0", "lnproc": "0", "mem_fault": "0", '
        expected_out += '"mem_usage": "0", "memphy": "0", "memphy_fault": "0", '
        expected_out += '"mep": "0", "mep_fault": "0", "ncpu": "0", "nproc": "0", "nproc_fault": "0", "reseller_id": "0" } ] }'

        expected = json.loads(expected_out)
        real = json.loads(out)
        self.assertCountEqual(expected['data'], real['data'])
        self.assertEqual(expected['hostname'], real['hostname'])
        self.assertEqual(expected['timestamp'], real['timestamp'])

    def test_multi_lve_data_set(self):
        self.t._set_timestamp('2014-02-06')

        t1_data = LVEStat(None, 10)
        t1_data.id = 11235813
        t1_data.cpu_usage = 23581321

        t2_data = LVEStat(None, 10)
        t2_data.id = 314159
        t2_data.mep = 265358

        t3_data = LVEStat(None, 10)
        t3_data.id = 271828
        t3_data.mep = 182845

        test_usage = {
            11235813: t1_data,
            314159: t2_data,
            271828: t3_data}

        out = self.t.format_json(test_usage)

        expected_out = '{ "hostname": "%s", "timestamp": "2014-02-06", ' % platform.node()
        expected_out += '"data": [ '
        expected_out += '{ "id": "271828", "cpu": "0", "cpu_usage": "0", '
        expected_out += '"io": "0", "io_usage": "0", "iops": "0", "lcpuw": "0", "lep": "0", "liops": "0", '
        expected_out += '"lmem": "0", "lmemphy": "0", "lnproc": "0", "mem_fault": "0", '
        expected_out += '"mem_usage": "0", "memphy": "0", "memphy_fault": "0", '
        expected_out += '"mep": "182845", "mep_fault": "0", "ncpu": "0", "nproc": "0", "nproc_fault": "0", "reseller_id": "0" }, '

        expected_out += '{ "id": "11235813", "cpu": "0", "cpu_usage": "23581321", '
        expected_out += '"io": "0", "io_usage": "0", "iops": "0", "lcpuw": "0", "lep": "0", "liops": "0", '
        expected_out += '"lmem": "0", "lmemphy": "0", "lnproc": "0", "mem_fault": "0", '
        expected_out += '"mem_usage": "0", "memphy": "0", "memphy_fault": "0", '
        expected_out += '"mep": "0", "mep_fault": "0", "ncpu": "0", "nproc": "0", "nproc_fault": "0", "reseller_id": "0" }, '

        expected_out += '{ "id": "314159", "cpu": "0", "cpu_usage": "0", '
        expected_out += '"io": "0", "io_usage": "0", "iops": "0", "lcpuw": "0", "lep": "0", "liops": "0", '
        expected_out += '"lmem": "0", "lmemphy": "0", "lnproc": "0", "mem_fault": "0", '
        expected_out += '"mem_usage": "0", "memphy": "0", "memphy_fault": "0", '
        expected_out += '"mep": "265358", "mep_fault": "0", "ncpu": "0", "nproc": "0", "nproc_fault": "0", "reseller_id": "0" } ] }'

        expected = json.loads(expected_out)
        real = json.loads(out)
        self.assertCountEqual(expected['data'], real['data'])
        self.assertEqual(expected['hostname'], real['hostname'])
        self.assertEqual(expected['timestamp'], real['timestamp'])
