#!/usr/bin/python
# coding=utf-8
from unittest import TestCase

import mock
from lvestat import LVEStat

from lvestats.plugins.generic.collectors import LVECollector

__author__ = "Aleksandr Shyshatsky"


class TestLVECollector(TestCase):
    def setUp(self):
        self.lve_collector = LVECollector()
        self.lve_collector.min_uid = 500

        patcher1 = mock.patch('lvestats.lib.config.read_config', return_value={})
        patcher1.start()
        self.addCleanup(patcher1.stop)

    def test_defaults(self):
        """Collector should save defaults from /proc/lve/list, NOT /proc/resellers/**/list"""
        def mock_parse_stats(lvp_id=0, without_limits=True):
            if lvp_id == 0:
                return [
                    '0	0	2000	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
                    '500	0	2500	1	1000001000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
                ]
            else:
                return [
                    '0	0	1234	1	1000000001	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
                    '501	0	2500	1	1000001000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0',
                ]

        data = {}
        with mock.patch('clcommon.clproc.ProcLve.lines', side_effect=mock_parse_stats):
            with mock.patch('clcommon.clproc.ProcLve.resellers_supported', return_value=True):
                with mock.patch('clcommon.clproc.ProcLve.version', return_value=8):
                    with mock.patch('clcommon.clproc.ProcLve.lvp_id_list', return_value=[501]):
                        self.lve_collector.execute(data)
        self.assertCountEqual([0, 500, 501], list(data['stats'].keys()))

        # pylint: disable=line-too-long
        self.assertCountEqual(data['stats'][0]._get_attributes(),
                              LVEStat('0	0	2000	1	1000000000	0	0	4096	20	1	0	0	4096	100	0	0	0	0	0   10   0', 8)._get_attributes())

    def test_limits(self):
        """Collector should save limits row from /proc/resellers/**/list
        (that row contains info about whole container load)"""
        def mock_parse_stats(lvp_id=0, without_limits=True):
            if lvp_id == 0:
                return []
            else:
                # pylint: disable=line-too-long
                return [
                    '1000,0          2500    100     1       20      100     0       262144  1024    1024    0       0       0       0       0       0       0       0       0       0       0       0       0       0       0',
                    '1000,1001          2500    100     1       20      100     0       262144  1024    1024    0       0       0       0       0       0       0       0       0       0       0       0       0       0       0',
                    '1000,4294967295          2500    100     1       20      100     0       262144  1024    1024    0       0       0       0       0       0       0       0       0       0       0       0       0       0       0'
                ]
        data = {}
        with mock.patch('clcommon.clproc.ProcLve.lines', side_effect=mock_parse_stats):
            with mock.patch('clcommon.clproc.ProcLve.resellers_supported', return_value=True):
                with mock.patch('clcommon.clproc.ProcLve.version', return_value=10):
                    with mock.patch('clcommon.clproc.ProcLve.lvp_id_list', return_value=[1000]):
                        self.lve_collector.execute(data)
        self.assertCountEqual([0, 1001, 1000001000], list(data['stats'].keys()))

        # pylint: disable=line-too-long
        self.assertCountEqual(data['stats'][1000001000]._get_attributes(),
                                      LVEStat('1000,4294967295          2500    100     1       20      100     0       262144  1024    1024    0       0       0       0       0       0       0       0       0       0       0       0       0       0       0', 10)._get_attributes())

