# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
import ddt
import mock
from lvestats.plugins.generic.lvestats_memory_usage import LVEStatsMemoryUsage


class MockProcess(object):
    def __init__(self, value):
        self._value = value

    def memory_info(self):
        return [self._value]


class MockLog(object):
    def __init__(self):
        self.message = None

    def debug(self, *args, **kwargs):
        pass

    def warning(self, *args, **kwargs):
        pass

    def error(self, *args, **kwargs):
        self.message = str(args) + str(kwargs)


class MockHeap(object):

    def __getitem__(self, item):
        if isinstance(item, slice):
            return self
        return super(MockHeap, self).__getitem__(item)

    def __str__(self):
        return "HEAP"

    def byid(self):
        return "BYID"

    def byvia(self):
        return "BYVIA"

    def byrcs(self):
        return "BYRCS"


class MockHpy(object):
    def setrelheap(self):
        pass

    def heap(self):
        return MockHeap()


@ddt.ddt
class TestLVEStatsMemoryUsage(unittest.TestCase):
    def test_init(self):
        with mock.patch("lvestats.plugins.generic.lvestats_memory_usage.hpy", return_value=MockHpy()):
            plugin = LVEStatsMemoryUsage()
        plugin.log = MockLog()
        self.assertNotEqual(plugin._process, None, msg=plugin.log.message)
        plugin.execute({})  # don't raise exeption

    def test_init_attribute_fail(self):
        with mock.patch("psutil.Process", side_effect=AttributeError):
            with mock.patch("lvestats.plugins.generic.lvestats_memory_usage.hpy", return_value=MockHpy()):
                plugin = LVEStatsMemoryUsage()
        self.assertEqual(plugin._process, None)
        plugin.execute({})  # don't raise exeption

    @ddt.data(
        [100, True],
        [1024 * 1024 * 512 - 1, True],
        [1024 * 1024 * 512 + 1, False])
    @ddt.unpack
    def test_execute(self, value, is_none):
        with mock.patch("lvestats.plugins.generic.lvestats_memory_usage.hpy", return_value=MockHpy()):
            plugin = LVEStatsMemoryUsage()
        plugin.log = MockLog()
        plugin._process = MockProcess(value)
        plugin.execute({})
        if is_none:
            self.assertIsNone(plugin.log.message)
        else:
            self.assertIsNotNone(plugin.log.message)
