#!/usr/bin/python
# coding=utf-8

import os
import re
import unittest
from contextlib import contextmanager
from copy import deepcopy

import mock
from clcommon import cpapi
from clcommon.cpapi import NotSupported
from ddt import ddt, data, unpack

from lvestats.lib.commons.func import merge_dicts
from lvestats.plugins.generic.statsnotifier import StatsNotifier
import test.testcases_data as testcases_data
from test.testcases_data import data_notify_customers, table_test_case1, table_test_case2, table_test_case3, \
    table_test_case_all_limits_faults, NO_SETTINGS, test_case_notify_customers, data_notify_resellers
from lvestats.lib.commons.dateutil import seconds_to_human_view


__author__ = "Aleksandr Shyshatsky"


template_test_cases = list()
#                           locale        variables for generating         variables extracted
#                           (directory    templates                        from generated body messages
#                           name)
template_test_cases.append(('en_US',     {'TONAME': 'Mr.President'},       {'TONAME': 'Mr.President'}))
template_test_cases.append(('en_US_old', {'TONAME': 'Mr.President'},       {'TONAME': 'Mr.President'}))

template_test_cases.append(('en_US',     {'DOMAIN': 'google.com'},         {'DOMAIN': 'google.com'}))
template_test_cases.append(('en_US_old', {'DOMAIN': 'google.com'},         {'DOMAIN': 'google.com'}))

template_test_cases.append(('en_US',     {'PERIOD': 60*60},             {'PERIOD': 60 * 60}))
template_test_cases.append(('en_US_old', {'PERIOD': 60*60},             {'PERIOD': 60 * 60}))

template_test_cases.append(('en_US',     {'epf': 1, 'lep': 20},            {'EPf': '1', 'lEP': '20'}))
template_test_cases.append(('en_US_old', {'EPf': 1, 'lEP': 20},            {'EPf': '1', 'lEP': '20'}))

template_test_cases.append(('en_US',     {'pmemf': 5, 'lpmem': 256},       {'PMemF': '5', 'lPMem': '256KB'}))
template_test_cases.append(('en_US_old', {'PMemF': 5, 'lPMem': 256*1024},  {'PMemF': '5', 'lPMem': '262.1 kB'}))

template_test_cases.append(('en_US',     {'vmemf': 7, 'lvmem': 1024},      {'VMemF': '7', 'lVMem': '1024KB'}))
template_test_cases.append(('en_US_old', {'VMemF': 7, 'lVMem': 1024*1024}, {'VMemF': '7', 'lVMem': '1.0 MB'}))

template_test_cases.append(('en_US',     {'nprocf': 3, 'lnproc': 20},      {'NprocF': '3', 'lNproc': '20'}))
template_test_cases.append(('en_US_old', {'NprocF': 3, 'lNproc': 20},      {'NprocF': '3', 'lNproc': '20'}))

template_test_cases.append(('en_US',     {'cpuf':  2, 'lcpu':  20},        {'CPUf': '2', 'lCPU': '20'}))
template_test_cases.append(('en_US_old', {'aCPU': 75, 'lCPU':  100},       {'lCPU': '100%'}))

template_test_cases.append(('en_US',     {'iof':  1, 'lio': 100},          {'IOf':      '1', 'lIO': '100KB/s'}))
template_test_cases.append(('en_US_old', {'aIO': 75, 'lIO': 100},          {'aIO': '75KB/s', 'lIO': '100KB/s'}))

template_test_cases.append(('en_US',     {'iopsf': 33, 'liops': 100},      {'IOPSf': '33', 'lIOPS': '100'}))
template_test_cases.append(('en_US_old', {'aIOPS': 75, 'lIOPS': 100},      {'aIOPS': '75', 'lIOPS': '100'}))


# patterns for extracting variables from generated templates
PATTERN_TONAME = re.compile("Dear (?P<TONAME>.+),")
PATTERN_DOMAIN_PERIOD = re.compile("Your (?P<DOMAIN>.+) web hosting account exceeded one or more of its resources "
                                   "within the last (?P<PERIOD>.+)\.")
PATTERN_EP_FAULT = re.compile("Exceeded the maximum of (?P<lEP>.+) concurrent website connections\. "
                              "Your website was not available (?P<EPf>.+) times because of this problem\.")
PATTERN_PMEM_FAULT = re.compile("Exceeded the physical memory limit of (?P<lPMem>.+)\. "
                                "Your website was not available (?P<PMemF>.+) times because of this problem\.")
PATTERN_VMEM_FAULT = re.compile("Exceeded the virtual memory limit of (?P<lVMem>.+)\. "
                                "Your website was not available (?P<VMemF>.+) times because of this problem\.")
PATTERN_NPROC_FAULT = re.compile("Exceeded the number of processes limit of (?P<lNproc>.+)\. "
                                 "Your website was not available (?P<NprocF>.+) times because of this problem\.")
PATTERN_NPROC_FAULT_OLD = re.compile("Exceeded the memory limit of (?P<lNproc>.+)\. "
                                     "Your website was not available (?P<NprocF>.+) times because of this problem\.")
PATTERN_CPU_FAULT = re.compile("You reached limit of (?P<lCPU>.+) of total server CPU usage (?P<CPUf>.+) times\. "
                               "Your website was forced to load slower to reduce its CPU usage\.")
PATTERN_CPU_FAULT_OLD = re.compile("Exceeded the CPU limit of (?P<lCPU>.+) of total server CPU\. "
                                   "Your website was forced to load slower to reduce its CPU usage\.")
PATTERN_IO_FAULT = re.compile("You reached limit of (?P<lIO>.+) disk io rate (?P<IOf>.+) times\. "
                              "The disk io speed for your account was slowed as a result of this problem\.")
PATTERN_IO_FAULT_OLD = re.compile("Exceeded the maximum disk io rate\. You used (?P<aIO>.+) of (?P<lIO>.+)\. "
                                  "The disk io speed for your account was slowed as a result of this problem\.")
PATTERN_IOPS_FAULT = re.compile("You reached limit of (?P<lIOPS>.+) I/O operations (?P<IOPSf>.+) times\. "
                                "The disk io speed for your account was slowed as a result of this problem\.")
PATTERN_IOPS_FAULT_OLD = re.compile("Exceeded the maximum of I/O operations\. "
                                    "You used (?P<aIOPS>.+) of (?P<lIOPS>.+) operations\. "
                                    "The disk io speed for your account was slowed as a result of this problem\.")
PATTERN_RESELLER_USERNAME = re.compile("(Your reseller account |Accounts that belong to )\"(?P<RESELLER_USERNAME>.+)\"")
PATTERN_ASCII_TABLE = re.compile(r'\n(=+.+\|\n=+)', re.DOTALL)
TEMPLATES_PATTERNS = [PATTERN_TONAME, PATTERN_DOMAIN_PERIOD, PATTERN_EP_FAULT, PATTERN_PMEM_FAULT, PATTERN_VMEM_FAULT,
                      PATTERN_NPROC_FAULT, PATTERN_CPU_FAULT, PATTERN_IO_FAULT, PATTERN_IOPS_FAULT,
                      PATTERN_CPU_FAULT_OLD, PATTERN_IO_FAULT_OLD, PATTERN_IOPS_FAULT_OLD, PATTERN_NPROC_FAULT_OLD,
                      PATTERN_RESELLER_USERNAME]
TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), 'data', 'emails_templates')
TEMPLATE_CUSTOM_DIR = os.path.join(os.path.dirname(__file__), 'data', 'emails_custom_templates')


@ddt
class TestStatsNotifierTemplates(unittest.TestCase):
    def setUp(self):
        # this variables every times present in generated templates
        self.templ_data_default = {'TONAME': 'Costumer', 'DOMAIN': 'google.com', 'PERIOD': 12*60*60}
        # this variables dynamicaly presents in templates
        self.templ_data_default_dyn = {
            'EPf':    0,  'aEP':    0, 'lEP':     20,
            'PMemF':  0,  'aPMem':  0, 'lPMem':  256,
            'VMemF':  0,  'aVMem':  0, 'lVMem':  256,
            'NprocF': 0,  'aNproc': 0, 'lNproc': 256,
            'CPUf':   0,  'aCPU':   0, 'lCPU':    20,
            'IOf':    0,  'aIO':    0, 'lIO':     20,
            'IOPSf':  0,  'aIOPS':  0, 'lIOPS':   20}

        # set empty config as default
        _mock_default_config = mock.patch(
            'lvestats.plugins.generic.statsnotifier.get_admin_notification',
            return_value=testcases_data.get_admin_settings())
        _mock_default_config.start()
        self.addCleanup(_mock_default_config.stop)

        # configure plugin for testing
        self.plugin_instance = StatsNotifier()
        self.plugin_instance.resellers = {'reseller1'}
        self.plugin_instance.email_admin = 'root@localhost'
        self.plugin_instance.TEMPLATE_DIR = TEMPLATE_DIR
        self.plugin_instance.TEMPLATE_CUSTOM_DIR = TEMPLATE_CUSTOM_DIR

        self._mock_resellers_supported = mock.patch.object(self.plugin_instance._proc_lve, 'resellers_supported',
                                                           return_value=False)
        self._mock_resellers_supported.start()
        self.addCleanup(self._mock_resellers_supported.stop)

        _mock_users_need_notification = mock.patch.object(self.plugin_instance._notifications,
                                                          'users_need_notification', return_value=True)
        _mock_users_need_notification.start()
        self.addCleanup(_mock_users_need_notification.stop)

        patcher = mock.patch('lvestats.lib.config.read_config', return_value={})
        patcher.start()
        self.addCleanup(patcher.stop)

    @contextmanager
    def mock_admin_settings(self, **kwargs):
        with mock.patch('lvestats.plugins.generic.statsnotifier.get_admin_notification',
                        return_value=testcases_data.get_admin_settings(**kwargs)) as mock_settings:
            self.plugin_instance._load_plugin_settings()
            yield mock_settings

    @data(*template_test_cases)
    @unpack
    def test_user_notify(self, locale_, input_, output_):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn, input_, {'LOCALE': locale_})
        output_vars = merge_dicts(self.templ_data_default, output_)
        period_secs = templ_data['PERIOD']
        _, text_body, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='user_notify.txt')
        extracted_variables = extract_variables(text_body)
        number, english_name = seconds_to_human_view(period_secs)
        output_vars['PERIOD'] = f'{number} {english_name}'
        self.assertDictEqual(output_vars, extracted_variables)

    @data(
        ('reseller_faults_notify.txt', {'RESELLER_USERNAME': 'Mr.Sell', 'LOCALE': 'en_US'}),
        ('reseller_notify.txt', {'RESELLER_USERNAME': 'Mr.Sell', 'LOCALE': 'en_US'}),
    )
    @unpack
    def test_reseller_notify(self, text_template_name, input_):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn, input_)
        _, text_body, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name=text_template_name)
        extracted_variables = extract_variables(text_body)
        self.assertEqual(input_['RESELLER_USERNAME'], extracted_variables['RESELLER_USERNAME'])

    @data(
        ('someuser', {'TOMAIL': 'reseller@neverexists.com', 'LOCALE': 'en_US', 'RESELLER_USERNAME': 'someuser'}),
    )
    @unpack
    def test_get_panel_reseller_data(self, reseller_login, output_):
        with mock.patch(
            'clcommon.cpapi.cpinfo',
            return_value=[('reseller@neverexists.com', 'en')],
        ):
            self.assertDictEqual(output_, self.plugin_instance.get_panel_reseller_data(reseller_login))

    @data([
        'en_US',
        [
            mock.call(os.path.join(TEMPLATE_DIR, 'en_US/user_notify.txt'),
                      templ_data={'LOCALE': 'en_US'}),
            mock.call(os.path.join(TEMPLATE_DIR, 'en_US/user_notify.html'),
                      templ_data={'LOCALE': 'en_US'})
        ]
    ], [
        'en_US_old',
        [
            mock.call(os.path.join(TEMPLATE_DIR, 'en_US_old/user_notify.txt'),
                      templ_data={'LOCALE': 'en_US_old'}),
            # html template does not exist -> send only text version
            # mock.call('en_US_old/user_notify.html',
            #           templ_data={'LOCALE': 'en_US_old'})
        ]
    ], [
        # silently fallback to en_us, because ru_EN does not exist
        'ru_EN',
        [
            mock.call(os.path.join(TEMPLATE_DIR, 'en_US/user_notify.txt'),
                      templ_data={'LOCALE': 'ru_EN'}),
            mock.call(os.path.join(TEMPLATE_DIR, 'en_US/user_notify.html'),
                      templ_data={'LOCALE': 'ru_EN'})
        ]
    ])
    @unpack
    def test_templates_fallback(self, locale_, calls_):
        templ_data = {'LOCALE': locale_}
        with mock.patch('lvestats.plugins.generic.statsnotifier.'
                        'clemail.ClEmail.generate_mail_jinja2', return_value=(None, None)) as mock_generate:
            _, _, _ = self.plugin_instance.generate_msg_body(
                templ_data=templ_data, text_templ_name='user_notify.txt',
                html_templ_name='user_notify.html')
        self.assertEqual(calls_, mock_generate.mock_calls)

    @data(['http://localhost:2083'],
          ['http://localhost:2222'],
          ['http://localhost/'])
    @unpack
    def test_panel_url_in_template(self, url):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn,
                                 {'LOCALE': 'en_US', 'LOGIN_URL': url})
        _, text_body, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='user_notify.txt')
        self.assertIn(url, text_body)

    def test_panel_url_not_supported(self):
        with mock.patch(
                'lvestats.plugins.generic.statsnotifier.get_user_login_url',
                side_effect=NotSupported('Test')):
            self.assertEqual(
                self.plugin_instance._get_panel_login_url('localhost'),
                'http://localhost/')

    def test_admin_with_hostname(self):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn,
                                 {'LOCALE': 'en_US', 'HOSTNAME': 'test.cloudlinux.com'})
        _, text_body, html_body = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='admin_notify.txt', html_templ_name='admin_notify.html')
        self.assertIn(' on host test.cloudlinux.com', text_body)
        self.assertIn(' on host test.cloudlinux.com', html_body)

    def test_admin_without_hostname(self):
        """
        Test for backward compatibility
        """
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn, {'LOCALE': 'en_US'})
        _, text_body, html_body = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='admin_notify.txt', html_templ_name='admin_notify.html')
        self.assertNotIn(' on host ', text_body)
        self.assertNotIn(' on host ', html_body)

    @data('user_notify.txt', 'user_notify2.txt', 'no_such_file.txt')
    def test_incorrect(self, text_templ_name):
        from lvestats.plugins.generic.statsnotifier import StatsNotifierTemplateError
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn,
                                 {'LOCALE': 'en_US_incorrect', 'LOGIN': 'user_name'})
        self.assertRaises(StatsNotifierTemplateError, self.plugin_instance.generate_msg_body,
                          templ_data=templ_data, text_templ_name=text_templ_name)

    def test_custom_subject(self):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn,
                                 {'LOCALE': 'en_EN_subject'})
        subject, _, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='custom_subject.txt')
        self.assertEqual('Some subject', subject)

    def test_custom_subject2(self):
        source_templ_data = merge_dicts(self.templ_data_default,
                                 self.templ_data_default_dyn,
                                 {'LOCALE': 'en_EN_subject',
                                  'TOMAIL': 'neo@matrix.com'})
        templ_data = deepcopy(source_templ_data)

        msg = self.plugin_instance.generate_msg(
            templ_data=templ_data, text_templ_name='without_subject.txt')
        self.assertEqual(
            msg['Subject'],
            '=?utf-8?q?Hosting_account_resources_exceeded?=')

        templ_data = deepcopy(source_templ_data)
        with self.mock_admin_settings(notify_subject='Customised hosting account resources exceeded'):
            msg = self.plugin_instance.generate_msg(templ_data=templ_data,
                                                    text_templ_name='without_subject.txt')
        self.assertEqual(
            msg['Subject'],
            '=?utf-8?q?Customised_hosting_account_resources_exceeded?=')

        templ_data = deepcopy(source_templ_data)
        msg = self.plugin_instance.generate_msg(templ_data=templ_data,
                                                text_templ_name='custom_subject.txt')
        self.assertEqual(
            msg['Subject'],
            '=?utf-8?q?Some_subject?=')

    @data('without_subject.txt', 'without_subject.txt', 'without_subject.txt')
    def test_without_subject(self, text_templ_name):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn,
                                 {'LOCALE': 'en_EN_subject'})
        subject, _, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name=text_templ_name)
        self.assertIsNone(subject)

    @data('en_US', 'en_EN_old', 'es_ES_old')
    def test_smooth_user_notify(self, locale):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn, {'LOCALE':  locale})
        _, _, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='user_notify.txt')

    @data('en_US', 'en_EN_old', 'es_ES_old')
    def test_smooth_admin_notify(self, locale):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn, {'LOCALE': locale})
        _, _, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='admin_notify.txt', html_templ_name='admin_notify.html')

    @data('en_US', 'en_EN_old', 'es_ES_old')
    def test_smooth_reseller_notify(self, locale):
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn, {'LOCALE': locale})
        _, _, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='reseller_notify.txt', html_templ_name='reseller_notify.html')

    @data(table_test_case1, table_test_case2)
    def test_check_admin(self, test_case):
        # mock for extract raw message to self._msg
        self._msg = ''

        def mock_sendmail(_, to_addrs, msg, encoding_name):
            """
            append customer emails to list
            :param msg:
            :return: None
            """
            self._msg = str(msg)

        self.plugin_instance.email_admin = 'admin@mail'
        self.plugin_instance.get_users_data = lambda fault_users_data: test_case.input[:]
        self.plugin_instance._mail_helper.sendmail = mock_sendmail

        def mock_lve_id_list(lvp_id=0):
            self.assertEqual(lvp_id, 0)
            return [1000, 1100, 2000]

        with \
                self.mock_admin_settings(notify_admin=True, notify_cpu=True, notify_mem=False, notify_nproc=False,
                                         notify_iops=False, notify_io=False, notify_ep=True), \
                mock.patch.object(self.plugin_instance, 'get_users_fault') as mock_get_fault, \
                mock.patch.object(self.plugin_instance._notifications,
                                  'admin_need_notification', return_value=True), \
                mock.patch.object(self.plugin_instance._proc_lve, 'lve_id_list', side_effect=mock_lve_id_list):
            self.plugin_instance._check_admin()
            self.assertEqual(len(mock_get_fault.call_args_list), 1)
            _, kwargs = mock_get_fault.call_args_list[0]
            self.assertEqual(kwargs['uids'], [1000, 1100, 2000])
        self.assertListEqual(test_case.output, parse_ascii_table(self._msg))

    @mock.patch("clcommon.cpapi.cpinfo")
    @data(table_test_case3)
    def test_check_reseller_locale(self, test_case, cpinfo):
        self._msg = ''

        def mock_sendmail(_, to_addrs, msg, encoding_name):
            """
            append customer emails to list
            :param msg:
            :return: None
            """
            self._msg = str(msg)

        # mock for extract raw message to self._msg
        cpinfo.return_value = [["john777@john.com", "ko"]]
        self.plugin_instance._mail_helper.sendmail = mock_sendmail
        with \
                mock.patch(
                    'lvestats.plugins.generic.statsnotifier.get_notification',
                    return_value=None
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_reseller_users_list_faults_data',
                    return_value=[[test_case.input[:], []]],
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_resellers_faults_data',
                    return_value=[]), \
                self.mock_admin_settings(notify_admin=True, notify_cpu=True, notify_mem=False,
                                         notify_nproc=False, notify_iops=False, notify_io=False,
                                         notify_ep=True):
            self.plugin_instance._check_resellers()
        self.assertListEqual(test_case.output, parse_ascii_table(self._msg))
        check_output = "한국어".encode("us-ascii", 'xmlcharrefreplace').decode("us-ascii")
        self.assertIn(check_output, self._msg)

    @data(table_test_case3)
    def test_check_reseller(self, test_case):
        self._msg = ''
        self.plugin_instance.send_msg = lambda msg: setattr(self, '_msg', str(msg))

        def _get_panel_reseller_data(reseller_login):
            email = {'reseller1': 'john777@john.com',
                     'robot': 'elliot@mr-robot.com'}[reseller_login]
            return dict(TOMAIL=email, LOCALE='en', RESELLER_USERNAME=reseller_login)

        with \
                mock.patch(
                    'lvestats.plugins.generic.statsnotifier.get_notification',
                    return_value=testcases_data.get_resellers_settings(
                        *(NO_SETTINGS,)
                    ),
                ), \
                mock.patch.object(
                    self.plugin_instance, 'get_panel_reseller_data',
                    side_effect=_get_panel_reseller_data,
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_reseller_users_list_faults_data',
                    return_value=[[test_case.input[:], []]],
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_resellers_faults_data',
                    return_value=[],
                ), \
                self.mock_admin_settings(notify_admin=True, notify_cpu=True, notify_mem=False,
                                         notify_nproc=False, notify_iops=False, notify_io=False,
                                         notify_ep=True):
            self.plugin_instance._check_resellers()

        match_dict = PATTERN_RESELLER_USERNAME.search(self._msg).groupdict()
        self.assertEqual(match_dict.get('RESELLER_USERNAME'), test_case.input[0]['LOGIN'])
        self.assertListEqual(test_case.output, parse_ascii_table(self._msg))

    @data(table_test_case_all_limits_faults)
    def test_check_reseller_all_limits_faults(self, test_case):
        self._msg = ''
        self.plugin_instance.send_msg = lambda msg: setattr(self, '_msg', str(msg))

        def _get_panel_reseller_data(reseller_login):
            email = {'reseller1': 'john777@john.com',
                     'robot': 'elliot@mr-robot.com'}[reseller_login]
            return dict(TOMAIL=email, LOCALE='en')

        with \
                mock.patch(
                    'lvestats.plugins.generic.statsnotifier.get_notification',
                    return_value=testcases_data.get_resellers_settings(
                        *(NO_SETTINGS,)
                    ),
                ), \
                mock.patch.object(
                    self.plugin_instance, 'get_panel_reseller_data',
                    side_effect=_get_panel_reseller_data,
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_reseller_users_list_faults_data',
                    return_value=[[test_case.input[:], []]],
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_resellers_faults_data',
                    return_value=[],
                ), \
                self.mock_admin_settings(notify_admin=True, notify_cpu=True, notify_mem=True,
                                         notify_nproc=True, notify_iops=True, notify_io=True,
                                         notify_ep=True):
            self.plugin_instance._check_resellers()
        self.assertListEqual(test_case.output, parse_ascii_table(self._msg))

    @data(*test_case_notify_customers)
    @unpack
    def test_notify_customers(self, notify_customer, notify_include_reseller_customer,
                              users_notification_settings, customers_emails):
        self._emails = []

        def mock_sendmail(_, to_addrs, msg, encoding_name):
            """
            append customer emails to list
            :param to_addrs:
            :param msg:
            :param encoding_name:
            :return: None
            """
            self._emails.append(to_addrs[0])

        def mock_should_be_notified(user):
            """
            returns user`s notification turned on/off setting
            """
            return users_notification_settings[user]

        self.plugin_instance._mail_helper.sendmail = mock_sendmail
        with \
                mock.patch(
                    'lvestats.plugins.generic.statsnotifier.get_notification',
                    return_value=testcases_data.get_resellers_settings(
                        NO_SETTINGS),
                ), \
                self.mock_admin_settings(
                    notify_cpu=True,
                    notify_customer=notify_customer,
                    notify_include_reseller_customer=notify_include_reseller_customer
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_customers_faults_data',
                    return_value=[data_notify_customers[:]],
                ), \
                mock.patch('lvestats.plugins.generic.statsnotifier.user_should_be_notified',
                           side_effect=mock_should_be_notified):
            self.plugin_instance._check_users()
        self.assertEqual(customers_emails, self._emails)

    def test_notify_resellers_about_customers_faults(self):
        _emails = []

        def mock_sendmail(_, to_addrs, msg, encoding_name):
            _emails.extend(to_addrs)

        with \
                mock.patch.object(
                    self.plugin_instance, 'prepare_resellers_data',
                    return_value=testcases_data.get_resellers_data(),
                ), \
                mock.patch.object(
                    self.plugin_instance._mail_helper, 'sendmail',
                    side_effect=mock_sendmail,
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_resellers_faults_data',
                    return_value=[]
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_reseller_users_list_faults_data',
                    return_value=[[data_notify_customers, [500]]]
                ):
            self.plugin_instance._check_resellers()
        self.assertEqual(['reseller1@reseller1.com'], _emails)

    def test_detect_admin_email_from_config(self):
        with self.mock_admin_settings(report_admin_email='some@email.com'):
            self.assertEqual(self.plugin_instance._admin_settings.REPORT_ADMIN_EMAIL, 'some@email.com')

    def test_detect_admin_email_no_config_no_panel_email(self):
        with mock.patch('lvestats.plugins.generic.statsnotifier.DEFAULT_ADMIN_MAIL', "admin@mail.com"):
            with mock.patch('clcommon.cpapi.get_admin_email', return_value=None):
                with self.mock_admin_settings(report_admin_email='',
                                              notify_from_email='notify_from@email.com'):
                    self.assertEqual(self.plugin_instance._detect_admin_email(), 'admin@mail.com')

    def test_detect_admin_email_no_config(self):
        with mock.patch('lvestats.plugins.generic.statsnotifier.DEFAULT_ADMIN_MAIL', return_value="root@localhost"):
            with mock.patch('clcommon.cpapi.get_admin_email', return_value='admin_controlpanel@email.com'):
                with self.mock_admin_settings(report_admin_email='',
                                              notify_from_email='notify_from@email.com'):
                    self.assertEqual(self.plugin_instance._detect_admin_email(), 'admin_controlpanel@email.com')

    def test_detect_admin_email_none(self):
        from lvestats.plugins.generic.statsnotifier import DEFAULT_ADMIN_MAIL
        with mock.patch('clcommon.cpapi.get_admin_email', return_value=None):
            with self.mock_admin_settings(report_admin_email='',
                                          notify_from_email=''):
                self.assertEqual(self.plugin_instance._detect_admin_email(), DEFAULT_ADMIN_MAIL)

    def test_detect_admin_email_no_config_cpapi_exception(self):
        with mock.patch('lvestats.plugins.generic.statsnotifier.DEFAULT_ADMIN_MAIL', "root@localhost"):
            with mock.patch('clcommon.cpapi.get_admin_email', side_effect=NotSupported('Ooops.. Unsupported panel')):
                cfg = {'notify_from_email': ''}
                self.plugin_instance.set_config(cfg)

                self.assertEqual(self.plugin_instance._detect_admin_email(), "root@localhost")

    def test_get_users_data_with_deleted_reseller(self):
        statsnotifier = StatsNotifier()
        userdata = {
                'ID': 501,
                'aCPU':   9.99,     'CPUf':   0,  'lCPU':   10.0,
                'aVMem':  131072.0, 'VMemF':  20, 'lVMem':  131072.0,
                'aPMem':  130000.0, 'PMemF':  0,  'lPMem':  131072.0,
                'aNproc': 99.99,    'NprocF': 0,  'lNproc': 100.0,
                'aIOPS':  1024.0,   'IOPSf':  0,  'lIOPS':  1024.0,
                'aIO':    1023.0,   'IOf':    0,  'lIO':    1024.0,
                'aEP':    1.0,      'EPf':    0,  'lEP':    20.0,
                                    'anyF':   20,
            }

        def mock_cpapi(cpuser=None, *a, **kw):  # pylint: disable=keyword-arg-before-vararg
            if cpuser == "robot":
                return [('elliot@mr-robot.com', 'whoismrrobot.com', 'en', 'deleted_reseller')]
            return []

        class MockPWD(object):
            def __init__(self, pw_name="robot", pw_gecos="Mr.Robot"):
                self.pw_name = pw_name
                self.pw_gecos = pw_gecos

        notify_from_email = statsnotifier._admin_settings.NOTIFY_FROM_EMAIL
        result = userdata.copy()
        result.update({'TONAME':   'Mr.Robot',                       'DOMAIN':   'whoismrrobot.com',    'HOSTNAME': statsnotifier.hostname,
                       'LOCALE':   'en_US',                          'LOGIN':    'robot',               'TOMAIL': 'elliot@mr-robot.com',
                       'FROMMAIL': notify_from_email,                'RESELLER': 'deleted_reseller',    'LOGIN_URL': 'http://link.panel',
                       'RESELLER': 'deleted_reseller'})
        with mock.patch("pwd.getpwuid", return_value=MockPWD()):
            with mock.patch.object(cpapi, "cpinfo", mock_cpapi):
                with mock.patch.object(statsnotifier, '_get_panel_login_url', return_value='http://link.panel'):
                    self.assertEqual(statsnotifier.get_users_data([userdata])[0], result)

    def test_notify_resellers_summary_faults(self):
        """Check notify reseller about his summary faults"""
        notified_users = ['reseller1@local.com', 'reseller2@local.com']
        self._emails = []

        def mock_sendmail(_, to_addrs, msg, encoding_name):
            """
            append customer emails to list
            :param msg:
            :return: None
            """
            self._emails.append(to_addrs[0])

        self.plugin_instance._mail_helper.sendmail = mock_sendmail

        def get_panel_reseller_data(reseller_login):
            return {'TOMAIL': "{}@local.com".format(reseller_login),
                    'LOCALE': 'en',
                    'RESELLER_USERNAME': reseller_login}

        with \
                mock.patch(
                    'lvestats.plugins.generic.statsnotifier.StatsNotifier.get_panel_reseller_data',
                    side_effect=get_panel_reseller_data
                ), \
                mock.patch.object(
                    self.plugin_instance, '_iter_resellers_faults_data',
                    return_value=[[data_notify_resellers, []]]), \
                mock.patch.object(
                    self.plugin_instance, '_iter_reseller_users_list_faults_data',
                    return_value=[]):
            self.plugin_instance._check_resellers()
        self.assertCountEqual(notified_users, self._emails)

    def test_prepare_user_template_data(self):
        user_data = {
            'aVMem': 1024,
            'aPMem': 2048,
            'lVMem': 512,
            'lPMem': 4096,
            'lIO': 256,
            'aCPU': 40
        }

        templ_data = self.plugin_instance._prepare_template_data(user_data)
        self.assertEqual({
            # old data format
            'aVMem': 1024 * 1024,
            'aPMem': 2048 * 1024,
            'lVMem': 512 * 1024,
            'lPMem': 4096 * 1024,
            'lIO': 256,
            'aCPU': 40,

            # new data format, mem should be in bytes
            'avmem': 1024,
            'apmem': 2048,
            'lvmem': 512,
            'lpmem': 4096,
            'lio': 256,
            'acpu': 40,
        }, templ_data)

    def test_notify_users_prepare_user_template_data(self):
        """Check whether _prepare_template_data is called while sending message to user"""
        dummy_template_data = dict(TOMAIL='customer')
        with mock.patch.object(self.plugin_instance, '_prepare_template_data',
                               return_value=dummy_template_data):
            with mock.patch.object(self.plugin_instance, 'send_notification') as mock_send_notification:
                users_data = [{'TOMAIL': 'smth'}]
                self.plugin_instance._notify_users(users_data)

                mock_send_notification.assert_called_with(
                    templ_data={'TOMAIL': 'customer'},
                    html_templ_name='user_notify.html',
                    text_templ_name='user_notify.txt')

    def test_process_resellers_summary_info_mails(self):
        """Check whether _prepare_template_data is called while sending message to reseller"""
        with mock.patch.object(self.plugin_instance, '_prepare_template_data',
                               return_value={'template': 'final_data'}) as mock_prepare_template_data:
            with mock.patch.object(self.plugin_instance, 'send_notification') as mock_send_notification:
                reseller_data = {'login': 'reseller1'}
                self.plugin_instance._notify_reseller_summary_faults(reseller_data)

                mock_prepare_template_data.assert_called_with(reseller_data)
                mock_send_notification.assert_called_with(
                    templ_data={'template': 'final_data'},
                    text_templ_name='reseller_faults_notify.txt',
                    html_templ_name='reseller_faults_notify.html')

    @data(
        ('admin', 'Administrator'),
        ('reseller', 'Reseller'),
        ('customer', 'Customer'),
    )
    @unpack
    def test_locales_json(self, user_type, user_name):
        # load test locale data
        locale_name = 'en_US_localized'
        templ_data = deepcopy(merge_dicts(self.templ_data_default, self.templ_data_default_dyn,
                                          {'LOCALE': locale_name}))
        templ_data['user_type'] = user_type
        test_locale_data = self.plugin_instance._load_locales_data(os.path.join(self.plugin_instance.TEMPLATE_DIR,
                                                                                locale_name))
        s_subject, text_body, _ = self.plugin_instance.generate_msg_body(
            templ_data=templ_data, text_templ_name='user_notify.txt')
        self.assertEqual(s_subject, test_locale_data['NOTIFY_FROM_SUBJECT'])
        self.assertIn(test_locale_data['PERIOD']['hours'], text_body)
        self.assertIn(user_name, text_body)

    @data(
        ('en_US', 'admin_notify.txt', (os.path.join(TEMPLATE_DIR, 'en_US'),
                                       os.path.join(TEMPLATE_DIR, 'en_US', 'admin_notify.txt'))),
        ('ru_RU', 'admin_notify.txt', (os.path.join(TEMPLATE_CUSTOM_DIR, 'ru_RU'),
                                       os.path.join(TEMPLATE_CUSTOM_DIR, 'ru_RU', 'admin_notify.txt'))),
    )
    @unpack
    def test__detect_path_for_file(self, _locale_name, _templ_filename, _expected_result):
        """
        Test for _detect_path_for_file function
        """
        file_dir, full_path_to_file = self.plugin_instance._detect_path_for_file(_locale_name, _templ_filename)
        self.assertEqual((file_dir, full_path_to_file), _expected_result)

    def test__detect_path_for_file_calls(self):
        """
        Test generate_msg_body should call _detect_path_for_file function
        """
        locale_name = 'en_US'
        text_template = 'admin_notify.txt'
        templ_data = merge_dicts(self.templ_data_default, self.templ_data_default_dyn, {'LOCALE': locale_name})
        with mock.patch.object(self.plugin_instance, '_detect_path_for_file',
                               side_effect=self.plugin_instance._detect_path_for_file) as _detect_mock:
            self.plugin_instance.generate_msg_body(templ_data=templ_data, text_templ_name=text_template,
                                                   html_templ_name='admin_notify.html')
            _detect_mock.assert_called_once_with(locale_name, text_template)


def extract_variables(text):
    """
    Extract variables from generated templates
    """
    variables_dict = dict()
    for pattern in TEMPLATES_PATTERNS:
        result = pattern.search(text)
        if result:
            variables_dict.update(result.groupdict())
    return variables_dict


def parse_ascii_table(text):
    output = list()
    header = None
    r = PATTERN_ASCII_TABLE.search(text)
    if r:
        text = r.group(1)
        for line in text.split('\n'):
            if '|' in line:
                cells = list(map(str.strip, line.split('|')))
                if header is None:
                    header = cells
                else:
                    row = cells
                    dict_ = dict(zip(header, row))
                    dict_.pop('', None)  # delete key '' if exist
                    output.append(dict_)
    return output

if __name__ == '__main__':
    unittest.main()
