# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest

from lvestat import LVEStat

from lvestats.plugins.other.res_mem_collector import ResMEMCollector


class TestResMEMCollector(unittest.TestCase):
    # Fields:
    # LVE	EP	lCPU	lIO	CPU	MEM	IO	lMEM	lEP	nCPU	fMEM	fEP	lMEMPHY	lCPUW	lNPROC	MEMPHY	fMEMPHY	NPROC	fNPROC  lIOPS   iOPS,
    proc_lve_list_v8 = [
        '2	0	2500	1024	1000000000	0	0	262144	20	1	0	0	262144	100	0	100	0	0	0   10   0',
        '3	0	2500	1024	1000000000	0	0	262144	20	1	0	0	262144	100	0	100	0	0	0   10   0',
        '4	0	2500	1024	1000000000	0	0	262144	20	1	0	0	262144	100	0	100	0	0	0   10   0',
        '6	0	2500	1024	1000000000	0	0	262144	20	1	0	0	0	100	0	100	0	0	0   10   0',
    ]

    def get_stats_v8(self):
        """
        :rtype: Dict[str, Dict[int, LVEStat]]
        """
        stats = {}
        for line in self.proc_lve_list_v8:
            stat = LVEStat(line, 8)
            stats[int(stat.__getattribute__('id'))] = stat
        return {'stats': stats}

    def test_update_lve_data(self):
        lve_data = self.get_stats_v8()
        pmem_dict = {2: 200, 3: 400000, 5: 300, 6: 400}
        ResMEMCollector.update_lve_data(lve_data, pmem_dict)
        stats = lve_data['stats']
        # test pmem was set
        self.assertEqual(stats[2].__getattribute__('memphy'), 200)
        # pmem > lMEMPHY, means that pMEM was set to lMEMPHY
        self.assertEqual(stats[3].__getattribute__('memphy'), 262144)
        # if we don't know about this id, we should set it to 0
        self.assertEqual(stats[4].__getattribute__('memphy'), 0)
        # if the ide wasn't present, we shouldn't update it
        self.assertNotIn(5, list(stats.keys()))
        # i lMEMPHY is 0, even if pmem > lMEMPHY, leave the number as it is
        self.assertEqual(stats[6].__getattribute__('memphy'), 400)

    def test_parse_mem_by_lve(self):
        data = '''        ID            MEM
     503              5
                        3
                        1
                        3
                        1
     502              1
                        1
                        1
'''
        pmem_dict = ResMEMCollector.parse_mem_per_lve(data.split('\n'))
        self.assertEqual(pmem_dict[503], 8*256)
        self.assertEqual(pmem_dict[502], 2*256)
