# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import os
import unittest
from uuid import uuid4

from mock import patch

import test.helpers as helpers
from lvestats.lib.server_id import UpdateOrCreateServerID
from test.base.base_db_test import BaseDbTest


class TestServerID(BaseDbTest):

    def setUp(self):
        super(TestServerID, self).setUp()
        self.config_path = '/tmp/{0}.cfg'.format(str(uuid4()))
        self.save_servers_line('localhost')
        self.fake_config('localhost')
        self.update_or_create_server_id = UpdateOrCreateServerID(
            self.engine,
            config_path=self.config_path
        )

    def fake_config(self, server_id):
        f = open(self.config_path, 'w')
        f.writelines(('# test comment\n', 'server_id={0}'.format(server_id)))
        f.close()

    def tearDown(self):
        super(TestServerID, self).tearDown()
        os.unlink(self.config_path)

    def test_resolve_localhost_auto(self):
        server_id = UpdateOrCreateServerID(
            self.engine,
            config_path=self.config_path
        ).auto()
        cfg_server_id = UpdateOrCreateServerID(
            self.engine,
            config_path=self.config_path
        )._get_current_server_id_cfg()
        self.assertEqual(server_id, cfg_server_id)

    @patch('lvestats.lib.server_id.input', return_value='www.example.com')
    def test_resolve_localhost_prompt(self, *args):
        with helpers.nostdout():
            server_id = self.update_or_create_server_id.prompt()
        self.assertEqual(server_id, 'www.example.com')
        # Change to the same value to test LVES-397
        with helpers.capture_all(self.update_or_create_server_id.prompt) as output:
            self.assertEqual('www.example.com', output["result"])
            self.assertIn("Server ID is already set to", output["stdout"])
        cfg_server_id = self.update_or_create_server_id._get_current_server_id_cfg()
        self.assertEqual(cfg_server_id, 'www.example.com')

    @patch('lvestats.lib.server_id.input', return_value='x' * 256)
    def test_resolve_localhost_prompt_255(self, *args):
        with helpers.capture_all(self.update_or_create_server_id.prompt) as output:
            self.assertIn("Server ID length should be less than 255", output["stdout"])
            self.assertEqual(None, output["result"])


if __name__ == '__main__':
    unittest.main()
