# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest

from sqlalchemy.sql.expression import select

from test.base.base_db_test import BaseDbTest
from lvestats.orm import incident
from lvestats.plugins.generic.snapshot_saver import SnapshotSaver


EMPTY_LVE_DATA = {'lve_usage': {}}


class SnapshotSaverLoadFromDbTestCase(BaseDbTest):

    def test_finalize_old_not_finished_incidents(self):
        # This one should not be finalized
        self.engine.execute(incident.__table__.insert(
            dict(uid=500, server_id='localhost', incident_start_time=5, incident_end_time=None, dump_time=500)))
        # This should be finalized, and set to 0
        self.engine.execute(incident.__table__.insert(
            dict(uid=600, server_id='localhost', incident_start_time=5, incident_end_time=None, dump_time=200)))

        now = 650
        snapshot_saver = SnapshotSaver()
        snapshot_saver.now = now
        snapshot_saver.set_config({'server_id': 'localhost'})
        snapshot_saver.set_db_engine(self.engine)

        snapshot_saver.init_session()

        self.assertEqual(list(self.engine.execute(select([incident.incident_end_time]))), [(None,), (200,)])


if __name__ == '__main__':
    unittest.main()
