# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
import unittest
import ddt
import mock
from lvestats.lib.chart import SvgChart


@ddt.ddt
class TestSvgChart(unittest.TestCase):
    def test_add_x_legend_with_one_point(self):
        """
        See LVES-249: https://cloudlinux.atlassian.net/browse/LVES-249
        """
        svg_chart = SvgChart()
        try:
            svg_chart._add_x_legend([1], number=5)
        except ZeroDivisionError as ex:
            self.fail(ex)

    @ddt.data(
        # database && average is empty -> draw label
        [[[(0.0, 0.0), (60.0, 0.0), (120.0, 0.0), (180.0, 0.0)],
          [(0.0, 1.0), (60.0, 1.0), (120.0, 1.0), (180.0, 1.0)],
          [(0.0, 0),   (60.0, 0),   (120.0, 0),   (180.0, 0)]],
         ['red', 'green', 'blue'],
         ['average', 'limit', 'database'],
         True],
        # database is not empty -> do not draw label
        [[[(0.0, 0.0), (60.0, 0.0), (120.0, 0.0), (180.0, 0.0)],
          [(0.0, 1.0), (60.0, 1.0), (120.0, 1.0), (180.0, 1.0)],
          [(0.0, 0),   (60.0, 0),   (120.0, 10),  (180.0, 0)]],
         ['red', 'green', 'blue'],
         ['average', 'limit', 'database'],
         False],
        # average is not empty -> do not draw label
        [[[(0.0, 9.0), (60.0, 0.0), (120.0, 0.0), (180.0, 0.0)],
          [(0.0, 1.0), (60.0, 1.0), (120.0, 1.0), (180.0, 1.0)],
          [(0.0, 0),   (60.0, 0),   (120.0, 0.0), (180.0, 0)]],
         ['red', 'green', 'blue'],
         ['average', 'limit', 'database'],
         False],
        # everything is empty, but no average lines -> do not draw label
        [[[(0.0, 0.0), (60.0, 0.0), (120.0, 0.0), (180.0, 0.0)],
          [(0.0, 0.0), (60.0, 0.0), (120.0, 0.0), (180.0, 0.0)],
          [(0.0, 0),   (60.0, 0),   (120.0, 0.0), (180.0, 0)]],
         ['red', 'green', 'blue'],
         ['cpuF', 'Af', 'Bf'],
         False]
    )
    @ddt.unpack
    def test_add_graph_no_activity_label(self, datasets, colors, names, result):
        svg_chart = SvgChart()
        with mock.patch.object(svg_chart, '_draw_no_activity') as mock_draw_no_activity:
            svg_chart.add_graph(datasets=datasets, colors=colors, names=names)
            self.assertEqual(mock_draw_no_activity.called, result)


if __name__ == '__main__':
    unittest.main(verbosity=3)
