# coding=utf-8
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
"""
This module should contain Tests-related constants, variables and functions
to generate various fake data and mocks for our tests.
It shouldn't contain any tests or testing logic.
"""


# Special value to emulate when Reseller has no personal settings specified
import collections

NO_SETTINGS = 'no_settings'


def get_resellers_data():
    return [{
        'STATS_HTML': u'',
        'STATS': u'',
        'USERS': ['john777', 'robot'],
        'LOCALE': 'en_US',
        'HOSTNAME': 'fake_hostname.com',
        'PERIOD': 12*60*60,
        'TOMAIL': 'reseller1@reseller1.com',
        'LOGIN': 'reseller1',
        'RESELLER_USERNAME': 'reseller1',
    }]


def get_resellers_settings(notify_reseller_on_customers_faults=True,
                           notify_customers_on_faults=True,
                           notify_reseller_on_total_faults=True,
                           notify_cpu=True,
                           notify_io=True,
                           notify_iops=True,
                           notify_mem=True,
                           notify_nproc=True,
                           notify_ep=True,
                           notify_reseller_period=5,
                           notify_customer_period=5,
                           ):
    if notify_reseller_on_customers_faults == NO_SETTINGS:     # JUST FIRST ARG
        return {}
    return {
        'faultsNotification': {
            'faultsToInclude':               {
                'concurrentConnections': notify_ep,
                'cpu':                   notify_cpu,
                'io':                    notify_io,
                'iops':                  notify_iops,
                'mem':                   notify_mem,
                'nproc':                 notify_nproc
            },
            'minimumNumberOfFaultsToNotify': {
                'customer': 1,
                'reseller': 1
            },
            'notify':                        {
                'customer': {
                    'period':     notify_customer_period,
                    'unitOfTime': 'hours'
                },
                'reseller': {
                    'period':     notify_reseller_period,
                    'unitOfTime': 'hours'
                }
            },
            'notifyResellerOnCustomers':   notify_reseller_on_customers_faults,
            'notifyCustomers':             notify_customers_on_faults,
            'notifyReseller':              notify_reseller_on_total_faults,
        }
    }


def get_admin_settings(notify_admin=True,
                       notify_customer=True,
                       notify_include_reseller_customer=True,
                       notify_reseller_on_customers_faults=True,

                       notify_ep=True,
                       notify_cpu=True,
                       notify_io=True,
                       notify_iops=True,
                       notify_mem=True,
                       notify_nproc=True,

                       notify_from_email='',
                       notify_subject='',
                       report_admin_email='',
                       notify_admin_period=12,
                       notify_user_period=3,
                       ):
    return {
        "faultsNotification": {
            "email": {
                "notifyFromEmail": notify_from_email,
                "notifySubject": notify_subject,
                "reportAdminMail": report_admin_email
            },
            "faultsToInclude": {
                "concurrentConnections": notify_ep,
                "cpu": notify_cpu,
                "io": notify_io,
                "iops": notify_iops,
                "mem": notify_mem,
                "nproc": notify_nproc
            },
            "minimumNumberOfFaultsToNotify": {
                "admin": 1,
                "user": 1
            },
            "notify": {
                "admin": {
                    "period": notify_admin_period,
                    "unitOfTime": "hours"
                },
                "user": {
                    "period": notify_user_period,
                    "unitOfTime": "hours"
                }
            },
            "notifyAdmin": notify_admin,
            "notifyCustomers": notify_customer,
            "notifyResellerCustomers": notify_include_reseller_customer,
            "notifyResellers": notify_reseller_on_customers_faults
        }
    }


data_notify_customers = [
    {
        'TONAME': 'Tom', 'DOMAIN': 'matrix.com', 'LOCALE': 'en_US', 'LOGIN': 'neo',
        'TOMAIL': 'neo@matrix.com', 'PERIOD': 12*60*60, 'RESELLER': 'root', 'id': 101,
        'aCPU':   0.39409, 'CPUf':   2, 'lCPU':   10.0,
        'aVMem':  7.871,   'VMemF':  0,  'lVMem':  131072.0,
        'aPMem':  17.962,  'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 0.081,   'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  0.001,   'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    35.214,  'IOf':    0,  'lIO':    1024.0,
        'aEP':    0.0,     'EPf':    0,  'lEP':    20.0,
                           'anyF':   2,
    }, {
        'TONAME': 'John', 'DOMAIN': 'john.com', 'LOCALE': 'en_US', 'LOGIN': 'john777',
        'TOMAIL': 'john777@john.com', 'PERIOD': 12*60*60, 'RESELLER': 'reseller1', 'id': 500,
        'aCPU':   0.39409, 'CPUf':   70, 'lCPU':   10.0,
        'aVMem':  7.871,   'VMemF':  0,  'lVMem':  131072.0,
        'aPMem':  17.962,  'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 0.081,   'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  0.001,   'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    35.214,  'IOf':    99,  'lIO':    1024.0,
        'aEP':    0.0,     'EPf':    0,  'lEP':    20.0,
                           'anyF':   140,
    }, {
        'TONAME': 'reseller1', 'DOMAIN': 'reseller1.com', 'LOCALE': 'en_US',
        'LOGIN': 'reseller1',
        'TOMAIL': 'reseller1@reseller1.com', 'PERIOD': 12*60*60, 'RESELLER': 'root',
        'id': 102,
        'aCPU': 0.39409, 'CPUf': 7, 'lCPU': 10.0,
        'aVMem': 7.871, 'VMemF': 0, 'lVMem': 131072.0,
        'aPMem': 17.962, 'PMemF': 0, 'lPMem': 131072.0,
        'aNproc': 0.081, 'NprocF': 0, 'lNproc': 100.0,
        'aIOPS': 0.001, 'IOPSf': 0, 'lIOPS': 1024.0,
        'aIO': 35.214, 'IOf': 0, 'lIO': 1024.0,
        'aEP': 0.0, 'EPf': 0, 'lEP': 20.0,
        'anyF': 2,
    }, {
        'TONAME': 'Mr.Robot', 'DOMAIN': 'whoismrrobot.com', 'LOCALE': 'en_US', 'LOGIN': 'robot',
        'FROMNAME': 'Elliot Alderson',
        'TOMAIL': 'elliot@mr-robot.com', 'PERIOD': 12*60*60, 'RESELLER': 'reseller1', 'id': 501,
        'aCPU':   9.99,      'CPUf':   5,   'lCPU':   10.0,
        'aVMem':  131072.0,  'VMemF':  20,  'lVMem':  131072.0,
        'aPMem':  130000.0,  'PMemF':  0,   'lPMem':  131072.0,
        'aNproc': 99.99,     'NprocF': 0,   'lNproc': 100.0,
        'aIOPS':  1024.0,    'IOPSf':  0,   'lIOPS':  1024.0,
        'aIO':    1023.0,    'IOf':    10,  'lIO':    1024.0,
        'aEP':    1.0,       'EPf':    0,   'lEP':    20.0,
                             'anyF':   20,
    }]

data_notify_resellers = [
    {
        'TONAME': 'Mr.Robot', 'LOCALE': 'en_US', 'LOGIN': 'reseller1',
        'FROMNAME': 'reseller1 Alderson',
        'PERIOD': 12*60*60, 'id': 1000000501,
        'aCPU':   120.,      'CPUf':   0,  'lCPU':   10.0,
        'aVMem':  123456.0,  'VMemF':  20, 'lVMem':  131072.0,
        'aPMem':  654331.0,  'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 100.0,     'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  1024.0,    'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    1023.0,    'IOf':    0,  'lIO':    1024.0,
        'aEP':    1.0,       'EPf':    0,  'lEP':    20.0,
                             'anyF':   20,
    }, {
        'TONAME': 'Mr.Robot', 'LOCALE': 'en_US', 'LOGIN': 'reseller2',
        'FROMNAME': 'reseller2 Alderson',
        'PERIOD': 12*60*60, 'id': 1000000502,
        'aCPU':   120.,      'CPUf':   0,  'lCPU':   10.0,
        'aVMem':  123456.0,  'VMemF':  20, 'lVMem':  131072.0,
        'aPMem':  654331.0,  'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 100.0,     'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  1024.0,    'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    1023.0,    'IOf':    0,  'lIO':    1024.0,
        'aEP':    1.0,       'EPf':    0,  'lEP':    20.0,
                             'anyF':   20,
    }
]


TableTestCase = collections.namedtuple("TableTestCase", ["input", "output"])
table_test_case1 = TableTestCase(
    input=[{
        'TONAME': 'John', 'DOMAIN': 'john.com', 'LOCALE': 'en_US', 'LOGIN': 'john777',
        'TOMAIL': 'john777@john.com', 'PERIOD': 12*60*60, 'RESELLER': 'root', 'id': 500,
        'aCPU':   0.39409, 'CPUf':   70, 'lCPU':   10.0,
        'aVMem':  7.871,   'VMemF':  0,  'lVMem':  131072.0,
        'aPMem':  17.962,  'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 0.081,   'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  0.001,   'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    35.214,  'IOf':    0,  'lIO':    1024.0,
        'aEP':    0.0,     'EPf':    0,  'lEP':    20.0,
                           'anyF': 140,
         }],

    output=[{'Username':      'john777',
             'Domain':        'john.com',
             'Name':          '---',
             'CPU Limit, %':  '10.0',
             'CPU Faults':    '70'}]
)
table_test_case2 = TableTestCase(
    input=[{
        'TONAME': 'John', 'DOMAIN': 'john.com', 'LOCALE': 'en_US', 'LOGIN': 'john777',
        'TOMAIL': 'john777@john.com', 'PERIOD': 12*60*60, 'RESELLER': 'root', 'id': 500,
        'aCPU':   0.39409, 'CPUf':   70, 'lCPU':   10.0,
        'aVMem':  7.871,   'VMemF':  0,  'lVMem':  131072.0,
        'aPMem':  17.962,  'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 0.081,   'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  0.001,   'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    35.214,  'IOf':    0,  'lIO':    1024.0,
        'aEP':    0.0,     'EPf':    0,  'lEP':    20.0,
                           'anyF':   140,
         }, {
        'TONAME': 'Mr.Robot', 'DOMAIN': 'whoismrrobot.com', 'LOCALE': 'en_US', 'LOGIN': 'robot',
        'FROMNAME': 'Elliot Alderson',
        'TOMAIL': 'elliot@mr-robot.com', 'PERIOD': 12*60*60, 'RESELLER': 'root', 'id': 501,
        'aCPU':   9.99,     'CPUf':   0,  'lCPU':   10.0,
        'aVMem':  131072.0, 'VMemF':  20, 'lVMem':  131072.0,
        'aPMem':  130000.0, 'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 99.99,    'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  1024.0,   'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    1023.0,   'IOf':    0,  'lIO':    1024.0,
        'aEP':    1.0,      'EPf':    0,  'lEP':    20.0,
                            'anyF':   20,
         }],

    output=[{'Username':       'john777',
             'Domain':         'john.com',
             'Name':           '---',
             'CPU Faults':     '70',
             'CPU Limit, %':   '10.0'},
            {'Username':       'robot',
             'Domain':         'whoismrrobot.com',
             'Name':           'Elliot Alderson',
             'CPU Limit, %':   '10.0',
             'CPU Faults':     '0'}
            ]
)
table_test_case3 = TableTestCase(
    input=[{
        'LOCALE': 'en_US', 'LOGIN': 'reseller1', 'TOMAIL': 'john777@john.com',
         }, {
        'TONAME': 'John', 'DOMAIN': 'john.com', 'LOCALE': 'en_US', 'LOGIN': 'john777',
        'PERIOD': 12*60*60, 'RESELLER': 'reseller1', 'id': 500,
        'aCPU':   0.39409, 'CPUf':   70, 'lCPU':   10.0,
        'aVMem':  7.871,   'VMemF':  0,  'lVMem':  131072.0,
        'aPMem':  17.962,  'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 0.081,   'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  0.001,   'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    35.214,  'IOf':    0,  'lIO':    1024.0,
        'aEP':    0.0,     'EPf':    0,  'lEP':    20.0,
                           'anyF':   140,
         }, {
        'TONAME': 'Mr.Robot', 'DOMAIN': 'whoismrrobot.com', 'LOCALE': 'en_US', 'LOGIN': 'robot',
        'FROMNAME': 'Elliot Alderson',
        'PERIOD': 12*60*60, 'RESELLER': 'reseller1', 'id': 501,
        'aCPU':   9.99,      'CPUf':   0,  'lCPU':   10.0,
        'aVMem':  131072.0,  'VMemF':  20, 'lVMem':  131072.0,
        'aPMem':  130000.0,  'PMemF':  0,  'lPMem':  131072.0,
        'aNproc': 99.99,     'NprocF': 0,  'lNproc': 100.0,
        'aIOPS':  1024.0,    'IOPSf':  0,  'lIOPS':  1024.0,
        'aIO':    1023.0,    'IOf':    0,  'lIO':    1024.0,
        'aEP':    1.0,       'EPf':    0,  'lEP':    20.0,
                             'anyF':   20,
         }],

    output=[{'Username':       'john777',
             'Domain':         'john.com',
             'Name':           '---',
             'CPU Faults':     '70',
             'CPU Limit, %':   '10.0'},
            {'Username':       'robot',
             'Domain':         'whoismrrobot.com',
             'Name':           'Elliot Alderson',
             'CPU Limit, %':   '10.0',
             'CPU Faults':     '0'}
            ]
)

table_test_case_all_limits_faults = TableTestCase(
    input=[{
        'TONAME': 'John', 'DOMAIN': 'john.com', 'LOCALE': 'en_US', 'LOGIN': 'john777',
        'PERIOD': 12*60*60, 'RESELLER': 'reseller1', 'id': 500,
        'aCPU':   0.39409, 'CPUf':   70, 'lCPU':   10.0,
        'aVMem':  7.871,   'VMemF':  10,  'lVMem':  131072.0,
        'aPMem':  17.962,  'PMemF':  50,  'lPMem':  131072.0,
        'aNproc': 0.081,   'NprocF': 60,  'lNproc': 100.0,
        'aIOPS':  0.001,   'IOPSf':  30,  'lIOPS':  1024.0,
        'aIO':    35.214,  'IOf':    60,  'lIO':    1024.0,
        'aEP':    0.0,     'EPf':    20,  'lEP':    20.0,
                           'anyF':   140,
         }],

    output=[{
        'Username': 'john777',
        'Domain': 'john.com',
        'EP Faults': '20',
        'Name': '---',
        'CPU Limit, %': '10.0',
        'PMem Limit, KB': '131072.0',
        'EP Limit': '20.0',
        'VMem Faults': '10',
        'NPROC Limit': '100.0',
        'NPROC Faults': '60',
        'IO Limit, KB/s': '1024.0',
        'IO Faults': '60',
        'CPU Faults': '70',
        'VMem Limit, KB': '131072.0',
        'PMem Faults': '50'
    }]
)


test_case_notify_customers = [
    # Columns:
    # 1. NOTIFY_CUSTOMER
    # 2. NOTIFY_RESELLER_CUSTOMERS
    # 3. Notify user settings
    # 4. RESULT

    # Cases with no per-reseller settings
    [True,
     True,
     {k['LOGIN']: True for k in data_notify_customers},
     ['neo@matrix.com', 'john777@john.com', 'reseller1@reseller1.com', 'elliot@mr-robot.com']],
    [True,
     False,
     {k['LOGIN']: True for k in data_notify_customers},
     ['neo@matrix.com', 'reseller1@reseller1.com']],  # reseller's customers not included
    [False,
     True,
     {k['LOGIN']: True for k in data_notify_customers},
     ['john777@john.com', 'elliot@mr-robot.com']],
    [False,
     False,
     {k['LOGIN']: True for k in data_notify_customers},
     []],
    [True,
     True,
     {'neo': False, 'john777': True, 'reseller1': True, 'robot': False},
     ['john777@john.com', 'reseller1@reseller1.com']]
]
